import pytest


@pytest.mark.django_db
def test_guardian_model_removed(migrator):
    migrator.apply_initial_migration(("portal", "0059_move_email_verifications_to_common"))
    new_state = migrator.apply_tested_migration(("portal", "0060_delete_guardian"))

    model_names = [model._meta.db_table for model in new_state.apps.get_models()]
    assert "portal_guardian" not in model_names



from __future__ import absolute_import

from selenium.webdriver.common.by import By

from .base_page import BasePage
from .home_page import HomePage


class PasswordResetPage(BasePage):
    def __init__(self, browser):
        super(PasswordResetPage, self).__init__(browser)

        assert self.on_correct_page("reset_password_page")

    def cancel(self):
        self.browser.find_element(By.ID, "cancel_button").click()
        return HomePage(self.browser)

    def reset_email_submit(self, email):
        self.browser.find_element(By.ID, "id_email").send_keys(email)

        self.wait_for_element_by_id("reset_button")

        self.browser.find_element(By.ID, "reset_button").click()
        return self



from django import template
from django.template.defaultfilters import floatformat

register = template.Library()


@register.filter(name="tableformat")
def tableformat(entry):

    if entry is None:
        return "-"
    elif is_numerical(entry):
        return floatformat(entry, -2)
    else:
        return entry


def is_numerical(str):
    try:
        float(str)
        return True
    except (ValueError, TypeError):
        return False



import common.permissions as permissions
from common.models import Class, School, Teacher
from django.contrib import messages as messages
from django.contrib.auth.decorators import login_required, user_passes_test
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.urls import reverse_lazy

from portal.forms.organisation import OrganisationForm


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(permissions.logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def organisation_create(request):
    teacher = request.user.new_teacher

    create_form = OrganisationForm(user=request.user)

    if request.method == "POST":
        create_form = OrganisationForm(request.POST, user=request.user)
        if create_form.is_valid():
            data = create_form.cleaned_data
            name = data.get("name", "")
            country = data.get("country")

            school = School.objects.create(name=name, country=country)

            teacher.school = school
            teacher.is_admin = True
            teacher.save()

            messages.success(request, "The school or club '" + teacher.school.name + "' has been successfully added.")

            return HttpResponseRedirect(reverse_lazy("onboarding-classes"))

    res = render(request, "portal/teach/onboarding_school.html", {"create_form": create_form, "teacher": teacher})

    return res


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(permissions.logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def organisation_manage(request):
    return organisation_create(request)


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(permissions.logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def organisation_leave(request):
    teacher = request.user.new_teacher

    check_teacher_is_not_admin(teacher)

    if request.method == "POST":
        classes = Class.objects.filter(teacher=teacher)
        for klass in classes:
            teacher_id = request.POST.get(klass.access_code, None)
            if teacher_id:
                new_teacher = get_object_or_404(Teacher, id=teacher_id)
                klass.teacher = new_teacher
                klass.save()

        classes = Class.objects.filter(teacher=teacher)
        teachers = Teacher.objects.filter(school=teacher.school).exclude(id=teacher.id)

        if classes.exists():
            messages.info(
                request,
                "You still have classes, you must first move them to another teacher within your school or club.",
            )
            return render(
                request,
                "portal/teach/teacher_move_all_classes.html",
                {
                    "original_teacher": teacher,
                    "classes": classes,
                    "teachers": teachers,
                    "submit_button_text": "Move classes and leave",
                },
            )

        teacher.school = None
        teacher.save()

        messages.success(request, "You have successfully left the school or club.")

        return HttpResponseRedirect(reverse_lazy("onboarding-organisation"))


def check_teacher_is_not_admin(teacher):
    if teacher.is_admin:
        raise Http404



from __future__ import absolute_import

import time

from selenium.webdriver.common.by import By

from portal.tests.pageObjects.portal.base_page import BasePage


class PlayBasePage(BasePage):
    def __init__(self, browser):
        super(PlayBasePage, self).__init__(browser)

    def logout(self):
        self.open_user_options_box()
        self.browser.find_element(By.ID, "logout_button").click()

        from portal.tests.pageObjects.portal.home_page import HomePage

        return HomePage(self.browser)

    def go_to_account_page(self):
        self.open_user_options_box()
        self.browser.find_element(By.ID, "student_edit_account_button").click()

        from .account_page import PlayAccountPage

        return PlayAccountPage(self.browser)

    def open_user_options_box(self):
        self.browser.find_element(By.ID, "logout_menu").click()
        time.sleep(1)



from __future__ import absolute_import

import datetime
from unittest.mock import ANY, Mock, patch

from common.mail import campaign_ids
from common.models import JoinReleaseStudent
from common.tests.utils import email as email_utils
from common.tests.utils.classes import create_class_directly
from common.tests.utils.organisation import (
    create_organisation_directly,
    join_teacher_to_organisation,
)
from common.tests.utils.student import (
    create_independent_student,
    create_independent_student_directly,
    create_school_student_directly,
    generate_independent_student_details,
    verify_email,
)
from common.tests.utils.teacher import signup_teacher_directly
from django.contrib.auth.models import User
from django.core import mail
from django.test import Client, TestCase
from django.urls import reverse
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait

from portal.forms.error_messages import INVALID_LOGIN_MESSAGE
from .base_test import BaseTest
from .pageObjects.portal.home_page import HomePage
from .utils.messages import (
    is_email_updated_message_showing,
    is_email_verified_message_showing,
    is_indep_student_join_request_received_message_showing,
    is_indep_student_join_request_revoked_message_showing,
    is_password_updated_message_showing,
    is_student_details_updated_message_showing,
)


class TestIndependentStudent(TestCase):
    def test_signup_short_password_fails(self):
        c = Client()

        response = c.post(
            reverse("register"),
            {
                "independent_student_signup-date_of_birth_day": 7,
                "independent_student_signup-date_of_birth_month": 10,
                "independent_student_signup-date_of_birth_year": 1997,
                "independent_student_signup-name": "Test Name",
                "independent_student_signup-email": "test@email.com",
                "independent_student_signup-consent_ticked": "on",
                "independent_student_signup-password": "pass",
                "independent_student_signup-confirm_password": "pass",
                "g-recaptcha-response": "something",
            },
        )

        # Assert response isn't a redirect (submit failure)
        assert response.status_code == 200

    def test_signup_common_password_fails(self):
        c = Client()

        response = c.post(
            reverse("register"),
            {
                "independent_student_signup-date_of_birth_day": 7,
                "independent_student_signup-date_of_birth_month": 10,
                "independent_student_signup-date_of_birth_year": 1997,
                "independent_student_signup-name": "Test Name",
                "independent_student_signup-email": "test@email.com",
                "independent_student_signup-consent_ticked": "on",
                "independent_student_signup-password": "Password1",
                "independent_student_signup-confirm_password": "Password1",
                "g-recaptcha-response": "something",
            },
        )

        # Assert response isn't a redirect (submit failure)
        assert response.status_code == 200

        response = c.post(
            reverse("register"),
            {
                "independent_student_signup-date_of_birth_day": 7,
                "independent_student_signup-date_of_birth_month": 10,
                "independent_student_signup-date_of_birth_year": 1997,
                "independent_student_signup-name": "Test Name",
                "independent_student_signup-email": "test@email.com",
                "independent_student_signup-consent_ticked": "on",
                "independent_student_signup-password": "Password123$",
                "independent_student_signup-confirm_password": "Password123$",
                "g-recaptcha-response": "something",
            },
        )
        assert response.status_code == 200

    def test_signup_passwords_do_not_match_fails(self):
        c = Client()

        response = c.post(
            reverse("register"),
            {
                "independent_student_signup-date_of_birth_day": 7,
                "independent_student_signup-date_of_birth_month": 10,
                "independent_student_signup-date_of_birth_year": 1997,
                "independent_student_signup-name": "Test Name",
                "independent_student_signup-email": "test@email.com",
                "independent_student_signup-consent_ticked": "on",
                "independent_student_signup-password": "Password1!",
                "independent_student_signup-confirm_password": "Password2!",
                "g-recaptcha-response": "something",
            },
        )

        # Assert response isn't a redirect (submit failure)
        assert response.status_code == 200

    def test_signup_invalid_name_fails(self):
        c = Client()

        response = c.post(
            reverse("register"),
            {
                "independent_student_signup-date_of_birth_day": 7,
                "independent_student_signup-date_of_birth_month": 10,
                "independent_student_signup-date_of_birth_year": 1997,
                "independent_student_signup-name": "///",
                "independent_student_signup-email": "test@email.com",
                "independent_student_signup-consent_ticked": "on",
                "independent_student_signup-password": "$RRFVBGT%6yhnmju7",
                "independent_student_signup-confirm_password": "$RRFVBGT%6yhnmju7",
                "g-recaptcha-response": "something",
            },
        )

        # Assert response isn't a redirect (submit failure)
        assert response.status_code == 200

    @patch("common.helpers.emails.send_dotdigital_email")
    def test_signup_under_13_sends_parent_email(self, mock_send_dotdigital_email: Mock):
        c = Client()

        response = c.post(
            reverse("register"),
            {
                "independent_student_signup-date_of_birth_day": datetime.date.today().day,
                "independent_student_signup-date_of_birth_month": datetime.date.today().month,
                "independent_student_signup-date_of_birth_year": datetime.date.today().year,
                "independent_student_signup-name": "Young person",
                "independent_student_signup-email": "test@email.com",
                "independent_student_signup-consent_ticked": "on",
                "independent_student_signup-password": "$RRFVBGT%6yhnmju7",
                "independent_student_signup-confirm_password": "$RRFVBGT%6yhnmju7",
                "g-recaptcha-response": "something",
            },
        )

        assert response.status_code == 302
        mock_send_dotdigital_email.assert_called_once_with(
            campaign_ids["verify_new_user_via_parent"], ANY, personalization_values=ANY
        )


# Class for Selenium tests. We plan to replace these and turn them into Cypress tests
class TestIndependentStudentFrontend(BaseTest):
    @patch("portal.views.registration.send_dotdigital_email")
    def test_delete_indy_account(self, mock_send_dotdigital_email: Mock):
        page = self.go_to_homepage()
        page, _, _, email, password = create_independent_student(page)
        page = page.independent_student_login(email, password)
        page = page.go_to_account_page()

        # save the user to check if it was anonymised
        user = User.objects.get(email=email)
        user_id = user.id

        # first check if a wrong password triggers the error
        unsubscribe_newsletter_checkbox = page.browser.find_element(By.NAME, "unsubscribe_newsletter")
        unsubscribe_newsletter_checkbox.click()

        delete_account_form = page.browser.find_element(By.NAME, "delete_password")
        delete_account_form.send_keys("123")  # wrong password

        delete_account_button = page.browser.find_element(By.ID, "delete_account_button")
        delete_account_button.click()
        assert (
            page.browser.find_element(By.CSS_SELECTOR, "#form-delete-indy-account > ul > li").text
            == "Incorrect password"
        )

        # now delete the account
        unsubscribe_newsletter_checkbox = page.browser.find_element(By.NAME, "unsubscribe_newsletter")
        unsubscribe_newsletter_checkbox.click()

        delete_account_form = page.browser.find_element(By.NAME, "delete_password")
        delete_account_form.send_keys(password)

        delete_account_button = page.browser.find_element(By.ID, "delete_account_button")
        delete_account_button.click()

        delete_button_confirm = page.browser.find_element(By.ID, "delete_button")
        delete_button_confirm.click()

        # check if can still login to the account
        page = self.go_to_homepage()
        assert page.go_to_independent_student_login_page().independent_student_login_failure(email, password)

        # now check if anonymised
        assert not User.objects.get(id=user_id).is_active

        # check if email has been sent
        mock_send_dotdigital_email.assert_called_once_with(campaign_ids["delete_account"], ANY)

    def test_signup_without_newsletter(self):
        page = self.go_to_homepage()
        page, _, _, _, _ = create_independent_student(page)
        assert is_email_verified_message_showing(self.selenium)

    @patch("portal.views.home.send_dotdigital_email")
    def test_signup_duplicate_email_failure(self, mock_send_dotdigital_email):
        page = self.go_to_homepage()
        page, _, _, email, _ = create_independent_student(page)
        assert is_email_verified_message_showing(self.selenium)

        page = self.go_to_homepage().go_to_signup_page()

        name, username, email_address, password = generate_independent_student_details()
        page = page.independent_student_signup(name, email, password=password, confirm_password=password)
        page = page.return_to_home_page()

        mock_send_dotdigital_email.assert_called_once_with(
            campaign_ids["user_already_registered"], ANY, personalization_values=ANY
        )

        login_link = mock_send_dotdigital_email.call_args.kwargs["personalization_values"]["LOGIN_URL"]

        page = email_utils.follow_duplicate_account_link_to_login(page, login_link, "independent")

        assert self.is_login_page(page)

    @patch("portal.views.home.send_dotdigital_email")
    def test_signup_duplicate_email_with_teacher(self, mock_send_dotdigital_email: Mock):
        teacher_email, _ = signup_teacher_directly()

        page = self.go_to_homepage()
        page = page.go_to_signup_page()

        strong_password = "£EDCVFR$5tgbnhy6"
        page = page.independent_student_signup(
            "indy", teacher_email, password=strong_password, confirm_password=strong_password
        )

        page.return_to_home_page()

        mock_send_dotdigital_email.assert_called_once_with(
            campaign_ids["user_already_registered"], ANY, personalization_values=ANY
        )

    def test_login_failure(self):
        page = self.go_to_homepage()
        page = page.go_to_independent_student_login_page()
        page = page.independent_student_login_failure("non-existent-email@codeforlife.com", "Incorrect password")

        assert page.has_login_failed("independent_student_login_form", INVALID_LOGIN_MESSAGE)

    def test_login_success(self):
        page = self.go_to_homepage()
        page, _, username, _, password = create_independent_student(page)
        page = page.independent_student_login(username, password)
        assert self.is_dashboard(page)

    @patch("common.helpers.emails.send_dotdigital_email")
    def test_login_not_verified(self, mock_send_dotdigital_email):
        username, password, _ = create_independent_student_directly(preverified=False)
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page = page.go_to_independent_student_login_page()
        page = page.independent_student_login_failure(username, password)

        page.has_login_failed("independent_student_login_form", INVALID_LOGIN_MESSAGE)
        assert page.has_login_failed("independent_student_login_form", INVALID_LOGIN_MESSAGE)

        verification_url = mock_send_dotdigital_email.call_args.kwargs["personalization_values"]["VERIFICATION_LINK"]

        verify_email(page, verification_url)

        assert is_email_verified_message_showing(self.selenium)

        page = page.independent_student_login(username, password)

        assert self.is_dashboard(page)

    @patch("portal.forms.registration.send_dotdigital_email")
    def test_reset_password(self, mock_send_dotdigital_email: Mock):
        page = self.go_to_homepage()

        page, name, username, _, _ = create_independent_student(page)
        page = self.get_to_forgotten_password_page()

        page.reset_email_submit(username)

        mock_send_dotdigital_email.assert_called_with(campaign_ids["reset_password"], ANY, personalization_values=ANY)

        reset_password_url = mock_send_dotdigital_email.call_args.kwargs["personalization_values"][
            "RESET_PASSWORD_LINK"
        ]

        page = email_utils.follow_reset_email_link(self.selenium, reset_password_url)

        new_password = "AnotherPassword12"

        page.student_reset_password(new_password)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_independent_student_login_page()
            .independent_student_login(username, new_password)
        )

        assert self.is_dashboard(page)

        page = page.go_to_account_page()
        assert page.check_account_details({"name": name})

    @patch("portal.forms.registration.send_dotdigital_email")
    def test_reset_password_fail(self, mock_send_dotdigital_email: Mock):
        page = self.get_to_forgotten_password_page()
        fake_email = "fake_email@fakeemail.com"
        page.reset_email_submit(fake_email)

        mock_send_dotdigital_email.assert_not_called()

    def test_update_name_success(self):
        homepage = self.go_to_homepage()

        play_page, name, student_username, _, password = create_independent_student(homepage)

        page = play_page.independent_student_login(student_username, password).go_to_account_page()

        assert page.check_account_details({"name": name})

        page = page.update_name_success("New name", password)

        assert self.is_dashboard(page)
        assert is_student_details_updated_message_showing(self.selenium)

    def test_update_name_failure(self):
        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(homepage)

        page = (
            play_page.independent_student_login(student_username, password)
            .go_to_account_page()
            .update_name_failure("Name!", password)
        )

        assert self.is_account_page(page)
        assert page.was_form_invalid(
            "student_account_form", "Names may only contain letters, numbers, dashes, underscores, and spaces."
        )

    @patch("common.helpers.emails.send_dotdigital_email")
    def test_change_email(self, mock_send_dotdigital_email):
        homepage = self.go_to_homepage()

        _, _, _, student_email, password = create_independent_student(homepage)
        play_page, _, _, other_email, _ = create_independent_student(homepage)

        page = play_page.independent_student_login(student_email, password).go_to_account_page()

        # Try changing email to an existing email, should fail
        page = page.change_email(other_email, password)
        assert self.is_email_verification_page(page)
        assert is_student_details_updated_message_showing(self.selenium)
        assert is_email_updated_message_showing(self.selenium)

        mock_send_dotdigital_email.assert_called_with(
            campaign_ids["email_change_notification"], ANY, personalization_values=ANY
        )

        # Try changing email to an existing teacher's email
        teacher_email, _ = signup_teacher_directly()

        page = (
            homepage.go_to_independent_student_login_page()
            .independent_student_login(student_email, password)
            .go_to_account_page()
        )

        page = page.change_email(teacher_email, password)
        assert self.is_email_verification_page(page)
        assert is_student_details_updated_message_showing(self.selenium)
        assert is_email_updated_message_showing(self.selenium)

        mock_send_dotdigital_email.assert_called_with(
            campaign_ids["email_change_notification"], ANY, personalization_values=ANY
        )

        page = (
            self.go_to_homepage()
            .go_to_independent_student_login_page()
            .independent_student_login(student_email, password)
            .go_to_account_page()
        )

        # Try changing email to a new one, should succeed
        new_email = "another-email@codeforlife.com"
        page = page.change_email(new_email, password)

        assert self.is_email_verification_page(page)
        assert is_student_details_updated_message_showing(self.selenium)
        assert is_email_updated_message_showing(self.selenium)

        # Check user can still log in with old account before verifying new email
        self.selenium.get(self.live_server_url)
        page = (
            self.go_to_homepage()
            .go_to_independent_student_login_page()
            .independent_student_login(student_email, password)
        )
        assert self.is_dashboard(page)

        page = page.logout()

        mock_send_dotdigital_email.assert_called_with(
            campaign_ids["email_change_verification"], ANY, personalization_values=ANY
        )
        verification_url = mock_send_dotdigital_email.call_args.kwargs["personalization_values"]["VERIFICATION_LINK"]

        page = email_utils.follow_change_email_link_to_independent_dashboard(page, verification_url)

        page = page.independent_student_login(new_email, password)

        assert self.is_dashboard(page)

    def test_change_password(self):
        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(homepage)

        page = play_page.independent_student_login(student_username, password).go_to_account_page()

        new_password = "AnotherPassword12"
        page = page.update_password_success(new_password, password, is_independent=True)

        assert self.is_login_page(page)
        assert is_student_details_updated_message_showing(self.selenium)
        assert is_password_updated_message_showing(self.selenium)

        page = page.independent_student_login(student_username, new_password)

        assert self.is_dashboard(page)

    def test_join_class_nonexistent_class(self):
        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(homepage)

        page = (
            play_page.independent_student_login(student_username, password)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club_failure("AA123")
        )

        assert self.is_join_class_page(page)
        assert page.has_join_request_failed(
            "The class code you entered either does not exist or is not currently accepting join requests. Please double check that you have entered the correct class code and contact the teacher of the class to ensure their class is currently accepting join requests."
        )

    def test_join_class_not_accepting_requests(self):
        teacher_email, _ = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        _, _, access_code = create_class_directly(teacher_email)
        create_school_student_directly(access_code)

        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(homepage)

        page = (
            play_page.independent_student_login(student_username, password)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club_failure(access_code)
        )

        assert self.is_join_class_page(page)
        assert page.has_join_request_failed(
            "The class code you entered either does not exist or is not currently accepting join requests. Please double check that you have entered the correct class code and contact the teacher of the class to ensure their class is currently accepting join requests."
        )

    def test_join_class_revoked(self):
        teacher_email, _ = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        klass, _, access_code = create_class_directly(teacher_email)
        create_school_student_directly(access_code)
        klass.always_accept_requests = True
        klass.save()

        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(homepage)

        page = (
            play_page.independent_student_login(student_username, password)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club(access_code)
        )

        assert is_indep_student_join_request_received_message_showing(self.selenium)

        page.revoke_join_request()

        assert is_indep_student_join_request_revoked_message_showing(self.selenium)

    def test_join_class_accepted(self):
        teacher_email, teacher_password = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        klass, _, access_code = create_class_directly(teacher_email)
        create_school_student_directly(access_code)
        klass.always_accept_requests = True
        klass.save()

        homepage = self.go_to_homepage()
        page = homepage.go_to_independent_student_login_page()

        username, password, student = create_independent_student_directly()

        page = (
            page.independent_student_login(username, password)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club(access_code)
        )

        page.logout()

        page = self.go_to_homepage()

        page = (
            page.go_to_teacher_login_page()
            .login(teacher_email, teacher_password)
            .open_classes_tab()
            .accept_independent_join_request()
            .save(username)
            .return_to_class()
        )

        assert page.student_exists(username)

        # check whether a record is created correctly
        logs = JoinReleaseStudent.objects.filter(student=student)
        assert len(logs) == 1
        assert logs[0].action_type == JoinReleaseStudent.JOIN

    def test_join_class_denied(self):
        teacher_email, teacher_password = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        klass, _, access_code = create_class_directly(teacher_email)
        create_school_student_directly(access_code)
        klass.always_accept_requests = True
        klass.save()

        homepage = self.go_to_homepage()

        play_page, _, student_username, _, password = create_independent_student(homepage)

        page = (
            play_page.independent_student_login(student_username, password)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club(access_code)
        )

        page.logout()

        page = self.go_to_homepage()

        dashboard_page = (
            page.go_to_teacher_login_page()
            .login(teacher_email, teacher_password)
            .open_classes_tab()
            .deny_independent_join_request()
        )

        assert dashboard_page.has_no_independent_join_requests()

    def test_join_class_denied_and_accepted_by_admin(self):
        # Create 2 teachers in the same school, one admin, one standard
        admin_email, admin_password1 = signup_teacher_directly()
        standard_email, _ = signup_teacher_directly()
        school = create_organisation_directly(admin_email)
        join_teacher_to_organisation(standard_email, school.name, is_admin=False)

        # Create class for standard teacher which always accepts external requests
        klass, class_name, access_code = create_class_directly(standard_email)
        klass.always_accept_requests = True
        klass.save()

        # Create two independent students
        username1, password1, student1 = create_independent_student_directly()
        username2, password2, student2 = create_independent_student_directly()

        # Login as both students and request to join the same class
        homepage = self.go_to_homepage()
        page = homepage.go_to_independent_student_login_page()

        page = (
            page.independent_student_login(username1, password1)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club(access_code)
        )

        page.logout()

        page = self.go_to_homepage()
        page = page.go_to_independent_student_login_page()

        page = (
            page.independent_student_login(username2, password2)
            .go_to_join_a_school_or_club_page()
            .join_a_school_or_club(access_code)
        )

        page.logout()

        # Login as school admin, accept the first request
        page = self.go_to_homepage()
        page = (
            page.go_to_teacher_login_page()
            .login(admin_email, admin_password1)
            .open_classes_tab()
            .accept_independent_join_request()
            .save(username1)
            .return_to_class()
        )

        assert page.student_exists(username1)

        # check whether a record is created correctly
        logs = JoinReleaseStudent.objects.filter(student=student1)
        assert len(logs) == 1
        assert logs[0].action_type == JoinReleaseStudent.JOIN

        # Deny the second request
        page = page.go_to_dashboard().open_classes_tab().deny_independent_join_request()

        assert page.has_no_independent_join_requests()

        # check a record hasn't been created and the student no longer has a join request
        logs = JoinReleaseStudent.objects.filter(student=student2)
        assert len(logs) == 0
        assert student2.pending_class_request is None
        assert student2.is_independent()

    def get_to_forgotten_password_page(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_independent_student_login_page().go_to_indep_forgotten_password_page()
        return page

    def wait_for_email(self):
        WebDriverWait(self.selenium, 2).until(lambda driver: len(mail.outbox) == 1)

    def is_dashboard(self, page):
        return page.__class__.__name__ == "PlayDashboardPage"

    def is_account_page(self, page):
        return page.__class__.__name__ == "PlayAccountPage"

    def is_join_class_page(self, page):
        return page.__class__.__name__ == "JoinSchoolOrClubPage"

    def is_login_page(self, page):
        return page.__class__.__name__ == "IndependentStudentLoginPage"

    def is_email_verification_page(self, page):
        return page.__class__.__name__ == "EmailVerificationNeededPage"



from django import template

register = template.Library()


@register.inclusion_tag("portal/partials/headline.html", takes_context=True)
def headline(context, headline_name):
    """
    Registers the inclusion tag for the headline partial.
    Takes in the name of the headline.
    The template currently expects the following context elements:
    - title
    - description
    """
    return context[headline_name]



import pytest


@pytest.mark.django_db
def test_models_moved_to_common(migrator):
    migrator.apply_initial_migration(("portal", "0057_delete_frontpagenews"))
    new_state = migrator.apply_tested_migration(("portal", "0058_move_to_common_models"))

    model_names = [model._meta.db_table for model in new_state.apps.get_models()]

    moved_models = ["userprofile", "school", "teacher", "class", "student"]

    for moved_model in moved_models:
        assert f"portal_{moved_model}" not in model_names
        assert f"common_{moved_model}" in model_names


@pytest.mark.django_db
def test_emailverification_moved_to_common(migrator):
    migrator.apply_initial_migration(("portal", "0058_move_to_common_models"))
    new_state = migrator.apply_tested_migration(("portal", "0059_move_email_verifications_to_common"))

    model_names = [model._meta.db_table for model in new_state.apps.get_models()]

    assert f"portal_emailverification" not in model_names
    assert f"common_emailverification" in model_names



TEACH_BANNER = {
    "title": "Educate",
    "subtitle": "Get ready to teach the next generation of computer scientists",
    "text": "",
    "image_class": "banner--picture--educate",
    "alt": "Female teacher helping young female student on a laptop",
}



from __future__ import absolute_import

from unittest.mock import patch

import pytest
from common.models import Class, School, Student, Teacher
from common.tests.utils.classes import create_class_directly
from common.tests.utils.organisation import (
    create_organisation_directly,
    join_teacher_to_organisation,
)
from common.tests.utils.student import create_school_student_directly
from common.tests.utils.teacher import signup_teacher_directly
from common.tests.utils.user import create_user_directly, get_superuser
from django.contrib.auth.models import User
from django.urls import reverse
from hamcrest import *
from hamcrest.core.base_matcher import BaseMatcher
from rest_framework import status
from rest_framework.test import APIClient, APITestCase


class APITests(APITestCase):
    def test_valid_date_registered(self):
        url = reverse(
            "registered-users",
            kwargs={"year": "2016", "month": "04", "day": "01"},
        )
        superuser = get_superuser()
        self.client.force_authenticate(user=superuser)
        response = self.client.get(url)
        assert_that(response, has_status_code(status.HTTP_200_OK))
        assert_that(isinstance(response.data, int))

    def test_invalid_date_registered(self):
        url = reverse(
            "registered-users",
            kwargs={"year": "2016", "month": "05", "day": "35"},
        )
        superuser = get_superuser()
        self.client.force_authenticate(user=superuser)
        response = self.client.get(url)
        assert_that(response, has_status_code(status.HTTP_404_NOT_FOUND))

    def test_valid_date_lastconnectedsince(self):
        url = reverse(
            "last-connected-since",
            kwargs={"year": "2016", "month": "04", "day": "01"},
        )
        superuser = get_superuser()
        self.client.force_authenticate(user=superuser)
        response = self.client.get(url)
        assert_that(response, has_status_code(status.HTTP_200_OK))
        assert_that(isinstance(response.data, int))

    def test_invalid_date_lastconnectedsince(self):
        url = reverse(
            "last-connected-since",
            kwargs={"year": "2016", "month": "05", "day": "35"},
        )
        superuser = get_superuser()
        self.client.force_authenticate(user=superuser)
        response = self.client.get(url)
        assert_that(response, has_status_code(status.HTTP_404_NOT_FOUND))

    def test_valid_country_userspercountry(self):
        url = reverse("number_users_per_country", kwargs={"country": "GB"})
        superuser = get_superuser()
        self.client.force_authenticate(user=superuser)
        response = self.client.get(url)
        assert_that(response, has_status_code(status.HTTP_200_OK))
        assert_that(isinstance(response.data, int))

    def test_get_inactive_users_if_admin(self):
        client = APIClient()
        superuser = get_superuser()
        create_user_directly(active=False)
        create_user_directly(active=True)
        client.force_authenticate(user=superuser)
        url = reverse("inactive_users")
        response = client.get(url)
        assert len(response.data) == 1

    @patch("portal.views.api.IS_CLOUD_SCHEDULER_FUNCTION", return_value=True)
    def test_get_inactive_users_if_appengine(
        self, mock_is_cloud_scheduler_function
    ):
        client = APIClient()
        create_user_directly(active=False)
        create_user_directly(active=True)
        url = reverse("inactive_users")
        response = client.get(url)
        assert mock_is_cloud_scheduler_function.called
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data) == 1

    def test_get_inactive_users_if_unauthorised(self):
        client = APIClient()
        create_user_directly(active=False)
        create_user_directly(active=True)
        url = reverse("inactive_users")
        response = client.get(url)
        assert response.status_code == status.HTTP_403_FORBIDDEN

    @patch("portal.views.api.IS_CLOUD_SCHEDULER_FUNCTION", return_value=True)
    def test_delete_inactive_users_if_appengine(
        self, mock_is_cloud_scheduler_function
    ):
        client = APIClient()
        create_user_directly(active=False)
        create_user_directly(active=False)
        url = reverse("inactive_users")
        response = client.get(url)
        users = response.data
        assert len(users) == 2

        # NOTE: Migration 0049 causes user 34 (created via migration 0001) to
        # be marked as inactive. Slightly tweaked this test so it still
        # passes but takes into account this new anonymisation.
        old_deleted_users = list(User.objects.filter(is_active=False))
        assert len(old_deleted_users) == 1

        response = client.delete(url)
        assert mock_is_cloud_scheduler_function.called
        assert response.status_code == status.HTTP_204_NO_CONTENT

        for user in users:
            with pytest.raises(User.DoesNotExist):
                User.objects.get(username=user["username"])

        deleted_users = list(User.objects.filter(is_active=False))
        new_deleted_users_count = len(deleted_users) - len(old_deleted_users)
        assert new_deleted_users_count == 2

        for user in deleted_users:
            assert user.first_name == "Deleted"
            assert user.last_name == "User"
            assert user.email == ""
            assert not user.is_active
            assert not client.login(username=user.username, password="password")
        response = client.get(url)
        assert len(response.data) == 0

    @patch("portal.views.api.IS_CLOUD_SCHEDULER_FUNCTION", return_value=True)
    def test_orphan_schools_and_classes_are_anonymised(
        self, mock_is_cloud_scheduler_function
    ):
        client = APIClient()
        # Create a school with an active teacher
        school1_teacher1_email, _ = signup_teacher_directly()
        school1 = create_organisation_directly(school1_teacher1_email)
        klass11, _, access_code11 = create_class_directly(
            school1_teacher1_email
        )
        _, _, student11 = create_school_student_directly(access_code11)

        # Create a school with one active non-admin teacher and one inactive admin teacher
        school2_teacher1_email, _ = signup_teacher_directly()
        school2_teacher2_email, _ = signup_teacher_directly()
        school2 = create_organisation_directly(school2_teacher1_email)
        join_teacher_to_organisation(
            school2_teacher2_email, school2.name, is_admin=True
        )
        klass21, _, access_code21 = create_class_directly(
            school2_teacher1_email
        )
        _, _, student21 = create_school_student_directly(access_code21)
        klass22, _, access_code22 = create_class_directly(
            school2_teacher2_email
        )
        _, _, student22 = create_school_student_directly(access_code22)
        school2_teacher1 = Teacher.objects.get(
            new_user__email=school2_teacher1_email
        )
        school2_teacher1.is_admin = False
        school2_teacher1.save()
        school2_teacher2 = Teacher.objects.get(
            new_user__email=school2_teacher2_email
        )
        school2_teacher2.new_user.is_active = False
        school2_teacher2.new_user.save()

        # Create a school with 2 inactive teachers
        school3_teacher1_email, _ = signup_teacher_directly()
        school3_teacher2_email, _ = signup_teacher_directly()
        school3 = create_organisation_directly(school3_teacher1_email)
        join_teacher_to_organisation(school3_teacher2_email, school3.name)
        klass31, _, access_code31 = create_class_directly(
            school3_teacher1_email
        )
        _, _, student31 = create_school_student_directly(access_code31)
        klass32, _, access_code32 = create_class_directly(
            school3_teacher2_email
        )
        _, _, student32 = create_school_student_directly(access_code32)
        school3_teacher1 = Teacher.objects.get(
            new_user__email=school3_teacher1_email
        )
        school3_teacher1.new_user.is_active = False
        school3_teacher1.new_user.save()
        school3_teacher2 = Teacher.objects.get(
            new_user__email=school3_teacher2_email
        )
        school3_teacher2.new_user.is_active = False
        school3_teacher2.new_user.save()

        # Create a school with no active teachers
        school4_teacher1_email, _ = signup_teacher_directly()
        school4 = create_organisation_directly(school4_teacher1_email)
        school4_teacher1 = Teacher.objects.get(
            new_user__email=school4_teacher1_email
        )
        school4_teacher1.new_user.is_active = False
        school4_teacher1.new_user.save()

        # Create a school with no teachers
        school5_teacher1_email, _ = signup_teacher_directly()
        school5 = create_organisation_directly(school5_teacher1_email)
        school5_teacher1 = Teacher.objects.get(
            new_user__email=school5_teacher1_email
        )
        school5_teacher1.delete()

        # Call the API
        url = reverse("anonymise_orphan_schools", kwargs={"start_id": 1})
        response = client.get(url)
        assert response.status_code == status.HTTP_204_NO_CONTENT

        # Check the first school/class/student still exist
        assert School.objects.filter(name=school1.name).exists()
        assert Class.objects.filter(pk=klass11.pk).exists()
        assert Student.objects.filter(pk=student11.pk).exists()

        # Check the second school exists and its first class/student, but the second ones are anonymised
        assert School.objects.filter(name=school2.name).exists()
        assert Class.objects.filter(pk=klass21.pk).exists()
        assert not Class.objects.filter(pk=klass22.pk).exists()
        assert Student.objects.filter(pk=student21.pk).exists()
        assert not Student.objects.get(pk=student22.pk).new_user.is_active
        # Also check the first teacher is now an admin
        assert Teacher.objects.get(
            new_user__email=school2_teacher1_email
        ).is_admin

        # Check the third school is anonymised together with its classes and students
        assert not School.objects.filter(name=school3.name).exists()
        assert not Class.objects.filter(pk=klass31.pk).exists()
        assert not Class.objects.filter(pk=klass32.pk).exists()
        assert not Student.objects.get(pk=student31.pk).new_user.is_active
        assert not Student.objects.get(pk=student32.pk).new_user.is_active

        # Check that the fourth school is anonymised
        assert not School.objects.filter(name=school4.name).exists()

        # Check that the fifth school is anonymised
        assert not School.objects.filter(name=school5.name).exists()

    def test_remove_fake_accounts(self):
        client = APIClient()
        initial_users_length = len(User.objects.all())
        admin_username = "codeforlife-portal@ocado.com"
        admin_password = "abc123"

        # First two accounts should be deleted
        # Third account should be omitted because first and last name is different
        # Fourth account should be omitted because it is verified
        random_accounts = [
            {
                "username": "hiya",
                "first_name": "name",
                "last_name": "name",
                "email": "eml@email.email",
                "password": '!QAZXSW"3edc',
                "verified": False,
            },
            {
                "username": "goodbye",
                "first_name": "hello",
                "last_name": "hello",
                "email": "el@email.email",
                "password": '!QAZXSW"3edc',
                "verified": False,
            },
            {
                "username": "different",
                "first_name": "nope",
                "last_name": "maybe",
                "email": "eail@email.email",
                "password": '!QAZXSW"3edc',
                "verified": False,
            },
            {
                "username": "same",
                "first_name": "lastname",
                "last_name": "lastname",
                "email": "eail@email.email",
                "password": '!QAZXSW"3edc',
                "verified": True,
            },
        ]

        for random_account in random_accounts:
            signup_teacher_directly(
                preverified=random_account["verified"],
                username=random_account["username"],
                email=random_account["email"],
                first_name=random_account["first_name"],
                last_name=random_account["last_name"],
            )

        assert (
            len(User.objects.all())
            == len(random_accounts) + initial_users_length
        )

        client.login(username=admin_username, password=admin_password)
        response = client.get(reverse("remove_fake_accounts"))
        assert response.status_code == 204

        # check if after deletion all the users are still there
        assert (
            len(User.objects.all()) == initial_users_length + 2
        )  # mentioned in the fake_accounts description


def has_status_code(status_code):
    return HasStatusCode(status_code)


class HasStatusCode(BaseMatcher):
    def __init__(self, status_code):
        self.status_code = status_code

    def _matches(self, response):
        return response.status_code == self.status_code

    def describe_to(self, description):
        description.append_text("has status code ").append_text(
            self.status_code
        )

    def describe_mismatch(self, response, mismatch_description):
        mismatch_description.append_text("had status code ").append_text(
            response.status_code
        )



from __future__ import absolute_import

from selenium.webdriver.common.by import By

from portal.tests.pageObjects.portal.email_verification_needed_page import EmailVerificationNeededPage
from portal.tests.pageObjects.portal.independent_login_page import IndependentStudentLoginPage
from portal.tests.pageObjects.portal.play.dashboard_page import PlayDashboardPage
from portal.tests.pageObjects.portal.student_login_class_code import StudentLoginClassCodePage
from .play_base_page import PlayBasePage


class PlayAccountPage(PlayBasePage):
    def __init__(self, browser):
        super(PlayAccountPage, self).__init__(browser)

        assert self.on_correct_page("play_account_page")

    def check_account_details(self, details):
        correct = True

        for field, value in list(details.items()):
            correct &= self.browser.find_element(By.ID, "id_" + field).get_attribute("value") == value

        return correct

    def _change_details(self, details):
        for field, value in list(details.items()):
            self.browser.find_element(By.ID, "id_" + field).clear()
            self.browser.find_element(By.ID, "id_" + field).send_keys(value)
        self.browser.find_element(By.ID, "update_button").click()

    def submit_empty_form(self):
        self.browser.find_element(By.ID, "update_button").click()
        return self

    def update_password_failure(self, new_password, confirm_new_password, old_password):
        self._update_password(new_password, confirm_new_password, old_password)
        return self

    def update_password_success(self, new_password, old_password, is_independent=False):
        self._update_password(new_password, new_password, old_password)
        if is_independent:
            return IndependentStudentLoginPage(self.browser)
        else:
            return StudentLoginClassCodePage(self.browser)

    def update_name_failure(self, new_name, password):
        self._update_name(new_name, password)
        return self

    def update_name_success(self, new_name, password):
        self._update_name(new_name, password)
        return PlayDashboardPage(self.browser)

    def change_email(self, new_email, password):
        self._change_details({"email": new_email, "current_password": password})
        return EmailVerificationNeededPage(self.browser)

    def _update_password(self, new_password, confirm_new_password, old_password):
        self.browser.find_element(By.ID, "id_password").send_keys(new_password)
        self.browser.find_element(By.ID, "id_confirm_password").send_keys(confirm_new_password)
        self.browser.find_element(By.ID, "id_current_password").send_keys(old_password)
        self.browser.find_element(By.ID, "update_button").click()

    def _update_name(self, new_name, password):
        self.browser.find_element(By.ID, "id_name").clear()
        self.browser.find_element(By.ID, "id_name").send_keys(new_name)
        self.browser.find_element(By.ID, "id_current_password").send_keys(password)
        self.browser.find_element(By.ID, "update_button").click()



"""
Patch SeleniumTestCase from django-selenium-clean package.

Instead of inheriting from StaticLiveServerTestCase, we inherit from LiveServerTestCase.
This solves a bug introduced when upgrading to Django 1.11,
see more information here: https://github.com/jazzband/django-pipeline/issues/593
"""

from django.contrib.staticfiles.testing import LiveServerTestCase
from django_selenium_clean import SeleniumTestCase

SeleniumTestCase.__bases__ = (LiveServerTestCase,)



MATERIALS_BANNER = {
    "title": "Rapid Router Teaching Packs",
    "subtitle": "A set of teaching materials to give you the confidence "
    "and resources to use Code for Life’s games with your students.",
    "text": "",
    "image_class": "banner--picture--tools",
}



import logging

from common.models import UserSession, Student, Class
from django.contrib import messages
from django.contrib.auth import authenticate, login
from django.contrib.auth.views import LoginView, FormView
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse_lazy
from django.utils.html import escape

from portal.forms.play import StudentLoginForm, StudentClassCodeForm
from portal.helpers.ratelimit import clear_ratelimit_cache_for_user
from portal.helpers.request_handlers import get_access_code_from_request
from portal.views.login import has_user_lockout_expired

LOGGER = logging.getLogger(__name__)


class StudentClassCodeView(FormView):
    template_name = "portal/login/student_class_code.html"
    form_class = StudentClassCodeForm
    extra_context = {"usertype": "SCHOOL_STUDENT"}

    def form_valid(self, form):
        self.form = form
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        class_code = self.form.cleaned_data["access_code"].upper()
        return reverse_lazy(
            "student_login",
            kwargs={"access_code": class_code, "login_type": "classform"},
        )


class StudentLoginView(LoginView):
    """View for login with class link"""

    template_name = "portal/login/student.html"
    form_class = StudentLoginForm
    success_url = reverse_lazy("student_details")
    extra_context = {"usertype": "SCHOOL_STUDENT"}

    def get_form_kwargs(self):
        kwargs = super(StudentLoginView, self).get_form_kwargs()
        kwargs["access_code"] = self.kwargs["access_code"].upper()
        return kwargs

    def _add_logged_in_as_message(self, request):
        class_name = self.kwargs["access_code"].upper()
        messages.info(
            request,
            f"<strong>You are logged in to class: "
            f"{escape(class_name)}</strong>",
            extra_tags="safe message--student",
        )

    def get_success_url(self):
        redirect_url = self.get_redirect_url()

        if redirect_url:
            return redirect_url

        self._add_logged_in_as_message(self.request)

        return self.success_url

    def _add_login_data(self, form, login_type):
        # class and student have been validated by this point
        class_code = self.kwargs["access_code"]
        classes = Class.objects.filter(access_code__iexact=class_code)
        klass = classes[0]

        name = form.cleaned_data.get("username")
        students = Student.objects.filter(
            new_user__first_name__iexact=name, class_field=klass
        )
        try:
            student = students[0]
        except IndexError:
            msg = f"Student {name} in class {class_code} is not found!"
            LOGGER.error(msg)
            raise Exception(msg)

        # Log the login time, class, and login type
        session = UserSession(
            user=student.new_user, class_field=klass, login_type=login_type
        )
        session.save()

        student.user.is_verified = True
        student.user.save()

    def form_valid(self, form):
        """Security check complete. Log the user in."""
        # Reset ratelimit cache upon successful login
        clear_ratelimit_cache_for_user(form.cleaned_data["username"])

        login_type = self.kwargs.get(
            "login_type", "classlink"
        )  # default to "classlink" if not specified

        self._add_login_data(form, login_type)
        return super(StudentLoginView, self).form_valid(form)

    def post(self, request, *args, **kwargs):
        """
        If the first name and access code found under the url inputted in the
        form corresponds to that of a blocked account, this redirects the user
        to the locked out page. However, if the lockout time is more than 24
        hours before this is executed, the account is unlocked.
        """
        username = request.POST.get("username")

        # get access code from the current url
        access_code = get_access_code_from_request(request)
        if Student.objects.filter(
            new_user__first_name=username, class_field__access_code=access_code
        ).exists():
            student = Student.objects.get(
                new_user__first_name=username,
                class_field__access_code=access_code,
            )

            if student.blocked_time is not None:
                if has_user_lockout_expired(student):
                    student.blocked_time = None
                    student.save()
                else:
                    return render(
                        self.request,
                        "portal/locked_out_school_student.html",
                        {"is_teacher": False},
                    )

        return super(StudentLoginView, self).post(request, *args, **kwargs)


def student_direct_login(request, user_id, login_id):
    """Direct login for student with unique url without username and password"""
    user = authenticate(request, user_id=user_id, login_id=login_id)

    if user:
        # Log the login time and class
        student = Student.objects.get(new_user=user)
        session = UserSession(
            user=user, class_field=student.class_field, login_type="direct"
        )
        session.save()

        login(request, user)

        student.user.is_verified = True
        student.user.save()

        return HttpResponseRedirect(reverse_lazy("student_details"))
    return HttpResponseRedirect(reverse_lazy("home"))



from __future__ import absolute_import

import time
from builtins import object

from selenium.common.exceptions import TimeoutException
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

FADE_TIME = 0.5


class BasePage(object):
    browser = None

    DEFAULT_WAIT_SECONDS = 5

    def __init__(self, browser):
        self.browser = browser

    def wait_for_element_by_id(self, id, wait_seconds=DEFAULT_WAIT_SECONDS):
        self.wait_for_presence((By.ID, id), wait_seconds)

    def wait_for_element_by_css(self, css, wait_seconds=DEFAULT_WAIT_SECONDS):
        self.wait_for_presence((By.CSS_SELECTOR, css), wait_seconds)

    def wait_for_element_to_be_clickable(
        self, locator, wait_seconds=DEFAULT_WAIT_SECONDS
    ):
        self.wait(EC.element_to_be_clickable(locator), wait_seconds)

    def wait_for_element_to_be_invisible(
        self, locator, wait_seconds=DEFAULT_WAIT_SECONDS
    ):
        self.wait(EC.invisibility_of_element_located(locator), wait_seconds)

    def wait_for_presence(self, locator, wait_seconds=DEFAULT_WAIT_SECONDS):
        self.wait(EC.presence_of_element_located(locator), wait_seconds)

    def wait_for_absence(self, locator, wait_seconds=DEFAULT_WAIT_SECONDS):
        self.wait_until_not(
            EC.presence_of_element_located(locator), wait_seconds
        )

    def wait(self, method, wait_seconds=DEFAULT_WAIT_SECONDS):
        WebDriverWait(self.browser, wait_seconds).until(method)

    def wait_until_not(self, method, wait_seconds=DEFAULT_WAIT_SECONDS):
        WebDriverWait(self.browser, wait_seconds).until_not(method)

    def element_exists(self, locator):
        try:
            self.wait_for_presence(locator)
            return True
        except TimeoutException:
            return False

    def element_does_not_exist(self, locator):
        try:
            self.wait_for_absence(locator)
            return True
        except TimeoutException:
            return False

    def element_does_not_exist_by_id(self, name):
        return self.element_does_not_exist((By.ID, name))

    def element_exists_by_id(self, name):
        return self.element_exists((By.ID, name))

    def element_does_not_exist_by_link_text(self, name):
        return self.element_does_not_exist((By.LINK_TEXT, name))

    def element_exists_by_css(self, name):
        return self.element_exists((By.CSS_SELECTOR, name))

    def on_correct_page(self, pageName):
        return self.element_exists_by_id(pageName)

    def hover_over_resources_dropdown(self):
        resources_dropdown = self.browser.find_element(
            By.ID, "teaching_resources_button"
        )
        hover = ActionChains(self.browser).move_to_element(resources_dropdown)
        hover.perform()

    def go_to_rapid_router_resources_page(self):
        self.hover_over_resources_dropdown()
        self.browser.find_element(
            By.ID, "rapid_router_resources_button"
        ).click()
        from .resources_page import ResourcesPage

        return ResourcesPage(self.browser)

    def is_on_admin_login_page(self):
        return self.on_correct_page("administration_login")

    def is_on_admin_data_page(self):
        return self.on_correct_page("admin_data")

    def is_on_admin_map_page(self):
        return self.on_correct_page("admin_map")

    def is_on_403_forbidden(self):
        return self.on_correct_page("403_forbidden")

    def was_form_invalid(self, formID, error):
        errors = (
            self.browser.find_element(By.ID, formID)
            .find_element(By.CLASS_NAME, "errorlist")
            .text
        )
        return error in errors

    def is_dialog_showing(self):
        time.sleep(FADE_TIME)
        return self.browser.find_element(By.ID, "popup").is_displayed()

    def confirm_dialog(self):
        self.browser.find_element(By.ID, "confirm_button").click()
        time.sleep(FADE_TIME)
        return self

    def cancel_dialog(self):
        self.browser.find_element(By.ID, "cancel_button").click()
        time.sleep(FADE_TIME)
        return self



from selenium.webdriver.common.by import By

from .base_page import BasePage
from .play import dashboard_page


class StudentLoginPage(BasePage):
    def __init__(self, browser):
        super(StudentLoginPage, self).__init__(browser)

        assert self.on_correct_page("student_login_page")

    def student_login(self, name, password):
        self._student_login(name, password)

        return dashboard_page.PlayDashboardPage(self.browser)

    def student_login_failure(self, name, password):
        self._student_login(name, password)
        return self

    def _student_login(self, name, password):
        self.browser.find_element(By.ID, "id_username").send_keys(name)
        self.browser.find_element(By.ID, "id_password").send_keys(password)
        self.browser.find_element(By.NAME, "school_login").click()

    def has_login_failed(self, form_id, error):
        errors = self.browser.find_element(By.ID, form_id).find_element(By.CLASS_NAME, "errorlist").text
        return error in errors



from __future__ import absolute_import

from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import Select

from . import email_verification_needed_page
from .base_page import BasePage


class SignupPage(BasePage):
    def __init__(self, browser):
        super(SignupPage, self).__init__(browser)

        assert self.on_correct_page("signup_page")

    def signup(
        self,
        first_name,
        last_name,
        email,
        password,
        confirm_password,
        success=True,
        newsletter=False,
    ):
        self.browser.find_element(By.ID, "id_teacher_signup-teacher_first_name").send_keys(first_name)
        self.browser.find_element(By.ID, "id_teacher_signup-teacher_last_name").send_keys(last_name)
        self.browser.find_element(By.ID, "id_teacher_signup-teacher_email").send_keys(email)
        self.browser.find_element(By.ID, "id_teacher_signup-teacher_password").send_keys(password)
        self.browser.find_element(By.ID, "id_teacher_signup-teacher_confirm_password").send_keys(confirm_password)
        self.browser.find_element(By.ID, "id_teacher_signup-consent_ticked").click()

        if newsletter:
            self.browser.find_element(By.ID, "id_teacher_signup-newsletter_ticked").click()

        self.browser.find_element(By.NAME, "teacher_signup").click()

        if success:
            return email_verification_needed_page.EmailVerificationNeededPage(self.browser)
        else:
            return self

    def independent_student_signup(self, name, email_address, password, confirm_password, success=True):
        dob_day_element = self.browser.find_element(By.ID, "id_independent_student_signup-date_of_birth_day")
        dob_month_element = self.browser.find_element(By.ID, "id_independent_student_signup-date_of_birth_month")
        dob_year_element = self.browser.find_element(By.ID, "id_independent_student_signup-date_of_birth_year")
        day_select = Select(dob_day_element)
        month_select = Select(dob_month_element)
        year_select = Select(dob_year_element)
        day_select.select_by_value("7")
        month_select.select_by_value("10")
        year_select.select_by_value("1997")
        self.browser.find_element(By.ID, "id_independent_student_signup-name").send_keys(name)
        self.browser.find_element(By.ID, "id_independent_student_signup-email").send_keys(email_address)
        self.browser.find_element(By.ID, "id_independent_student_signup-password").send_keys(password)
        self.browser.find_element(By.ID, "id_independent_student_signup-confirm_password").send_keys(confirm_password)
        self.browser.find_element(By.ID, "id_independent_student_signup-consent_ticked").click()

        self.browser.find_element(By.NAME, "independent_student_signup").click()
        if success:
            from .email_verification_needed_page import EmailVerificationNeededPage

            return EmailVerificationNeededPage(self.browser)
        else:
            return self



from django.contrib import messages as messages
from django.contrib.auth import logout
from django.contrib.auth.views import (
    PasswordChangeView,
    PasswordChangeDoneView,
)
from django.http import HttpResponseRedirect
from rest_framework.reverse import reverse_lazy

from portal.forms.admin import AdminChangeOwnPasswordForm

block_limit = 5


class AdminChangePasswordView(PasswordChangeView):
    form_class = AdminChangeOwnPasswordForm
    success_url = reverse_lazy("administration_password_change_done")

    def form_valid(self, form):
        return super(AdminChangePasswordView, self).form_valid(form)


class AdminChangePasswordDoneView(PasswordChangeDoneView):
    def get(self, request, *args, **kwargs):
        logout(request)
        messages.success(
            request,
            "Password updated successfully. Please login using your new password.",
        )
        return HttpResponseRedirect(reverse_lazy("teacher_login"))



from __future__ import absolute_import

from selenium.webdriver.common.by import By

from portal.tests.pageObjects.portal.play.join_school_or_club_page import JoinSchoolOrClubPage
from .play_base_page import PlayBasePage


class PlayDashboardPage(PlayBasePage):
    def __init__(self, browser):
        super(PlayDashboardPage, self).__init__(browser)

        assert self.on_correct_page("play_dashboard_page")

    def go_to_join_a_school_or_club_page(self):
        self.browser.find_element(By.ID, "student_join_school_link").click()

        return JoinSchoolOrClubPage(self.browser)



from __future__ import absolute_import

from selenium.webdriver.common.by import By

from . import home_page
from .base_page import BasePage


class EmailVerificationNeededPage(BasePage):
    def __init__(self, browser):
        super(EmailVerificationNeededPage, self).__init__(browser)
        assert self.on_correct_page("emailVerificationNeeded_page")

    def return_to_home_page(self):
        self.browser.find_element(By.ID, "home_button").click()
        return home_page.HomePage(self.browser)



from django.test import Client
from django.urls import reverse
from portal.tests.base_test import BaseTest
from portal.tests.pageObjects.portal.teacher_login_page import TeacherLoginPage
from portal.views import admin
import pytest


class TestAdmin(BaseTest):
    @classmethod
    def setUpClass(cls):
        super(TestAdmin, cls).setUpClass()
        admin.block_limit = 100

    # NB: Users are not expected to navigate to admin login page directly
    def navigate_to_admin_login(self):
        url = self.live_server_url + reverse("teacher_login")
        self.selenium.get(url)
        return TeacherLoginPage(self.selenium)

    # Checks all admin pages goes to admin_login when user is not logged in
    def test_navigate_to_admin_login(self):
        self.navigate_to_admin_login()


@pytest.mark.django_db
def test_export_user_data():
    admin_username = "codeforlife-portal@ocado.com"
    admin_password = "abc123"
    expected_data = ["indianajones@codeforlife.com", "Indiana", "Jones", "indianajones@codeforlife.com"]

    c = Client()
    c.login(username=admin_username, password=admin_password)

    url = reverse("admin:auth_user_changelist")
    data = {"action": "export_as_csv", "select_across": 0, "index": 0, "_selected_action": 11}

    response = c.post(url, data)
    csv_data = response.getvalue().decode("utf-8").split(",")

    assert any(item in expected_data for item in csv_data)
    assert "password" not in csv_data



from common.tests.utils.classes import create_class_directly
from common.tests.utils.organisation import create_organisation_directly
from common.tests.utils.student import create_school_student_directly
from common.tests.utils.teacher import signup_teacher_directly
from django.contrib.auth.models import User
from django.test import Client, TestCase
from django.test.utils import override_settings
from django.conf import settings
from django.urls import reverse

from django_otp.util import random_hex
from django_otp.oath import totp
from django_otp import DEVICE_ID_SESSION_KEY, user_has_device


class Test2FA(TestCase):
    """
    This tests our use of two factor authentication using django-two-factor-auth and django-otp modules.

    See https://tools.ietf.org/html/rfc6238#section-5.2 for detailed specs on TOTP.
    """

    def setUp(self) -> None:
        self.client = Client()
        self.email, self.password = self._setup_user()

    def tearDown(self) -> None:
        pass

    def _setup_user(self) -> (str, str):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        return email, password

    @override_settings(LOGIN_REDIRECT_URL="/teach/dashboard/")
    def test_token(self):
        def _post(data=None):
            return self.client.post("/login/teacher/", data=data)

        user = User.objects.get(email=self.email)

        # In production tolerance value is 1 by default, which means it will accept any of three
        # tokens: the current one, the previous one, and the next one.
        device = user.totpdevice_set.create(name="default", key=random_hex(), tolerance=0)

        # For us the authentication is done in 2 steps. First step is 'auth'.
        response = _post(
            {"auth-username": self.email, "auth-password": self.password, "teacher_login_view-current_step": "auth"}
        )
        self.assertContains(response, "Token:")

        # Second step is 'token'. There's a time window when these 2 steps must be done.
        #
        # Test 1: test random token
        response = _post({"token-otp_token": "123456", "teacher_login_view-current_step": "token"})
        INVALID_ERR = {"__all__": ["Invalid token. Please make sure you have entered it correctly."]}
        assert response.context_data["wizard"]["form"].errors == INVALID_ERR

        # reset throttle to allow us to continue after failure
        device.throttle_reset()

        # Test 2: test with 1 token before (drift = -1 which would fail with tolerance = 0)
        # In production, drift of [-1, 0, 1] should work with tolerance = 1
        response = _post(
            {"token-otp_token": totp(device.bin_key, drift=-1), "teacher_login_view-current_step": "token"}
        )
        assert response.context_data["wizard"]["form"].errors == INVALID_ERR

        # reset throttle to allow us to continue after failure
        device.throttle_reset()

        # Test 3: test with valid current token (drift = 0)
        response = _post({"token-otp_token": totp(device.bin_key, drift=0), "teacher_login_view-current_step": "token"})

        self.assertRedirects(response, settings.LOGIN_REDIRECT_URL, fetch_redirect_response=False)

        assert device.persistent_id == self.client.session.get(DEVICE_ID_SESSION_KEY)

        # test delete 2FA
        url = reverse("teacher_disable_2FA", kwargs={"pk": user.new_teacher.id})
        self.client.post(url)

        assert not user_has_device(user)



ABOUT_BANNER = {
    "title": "About Code for Life",
    "subtitle": "Code For Life gives everyone the ability to shape technology’s future",
    "text": "",
    "image_class": "banner--picture--about",
}

GETINVOLVED_BANNER = {
    "title": "Get involved",
    "subtitle": "How you can get involved with the creation of Code for Life products and resources",
    "image_class": "banner--picture--getinvolved",
    "button": {
        "external": True,
        "text": "Opportunities with Code for Life",
        "link": "https://2662351606-files.gitbook.io/~/files/v0/b/gitbook-x-prod.appspot.com/o/spaces%2F-M_6UDUiGoJDhr_OXetO%2Fuploads%2FPavvCifcuUeZZZLkV01C%2FOpportunities%20with%20Code%20for%20Life.pdf?alt=media",
    },
}

CONTRIBUTE_BANNER = {
    "title": "Contribute to Code for Life",
    "subtitle": "How you can get involved with Code for Life products and resources and help us help others",
    "image_class": "banner--picture--contribute",
}



from __future__ import absolute_import

from selenium.webdriver.common.by import By

from . import class_page
from .teach_base_page import TeachBasePage


class TeachMoveStudentsDisambiguatePage(TeachBasePage):
    def __init__(self, browser):
        super(TeachMoveStudentsDisambiguatePage, self).__init__(browser)

        assert self.on_correct_page("move_students_disambiguate_page")

    def cancel(self):
        self.browser.find_element(By.ID, "cancel_button").click()
        return class_page.TeachClassPage(self.browser)

    def move(self):
        self.browser.find_element(By.ID, "move_button").click()
        return class_page.TeachClassPage(self.browser)



HELP_BANNER = {
    "title": "Help and support",
    "subtitle": "",
    "text": "If you’ve run into problems, please look at our FAQs "
    "below for advice to commonly asked questions. If you still can’t find the "
    "answers, please contact us. One of the Code for Life team "
    "will get back to you and resolve your issues.",
    "image_class": "banner--picture--help",
}



TEN_YEAR_MAP_BANNER = {
    "title": "Celebrate",
    "subtitle": "Join us in celebrating our 10 year anniversary",
    "text": "",
    "image_class": "banner--picture--educate",
    "alt": "Female teacher helping young female student on a laptop",
}

TEN_YEAR_MAP_HEADLINE = {
    "title": "Code for Life is turning 10!",
    "description": "For our 10th anniversary, we're going global! Click on the pins on the map "
    "to check out what events are being run by our amazing volunteers from around the world.",
}



from datetime import timedelta
from time import sleep
from uuid import uuid4

import pytest
from common.models import SchoolTeacherInvitation, Teacher
from common.tests.utils.classes import create_class_directly
from common.tests.utils.organisation import create_organisation_directly
from common.tests.utils.teacher import signup_teacher_directly
from django.contrib.messages import get_messages
from django.core import mail
from django.test import Client, TestCase
from django.urls import reverse
from django.utils import timezone
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from portal.tests.base_test import BaseTest, click_buttons_by_id

FADE_TIME = 0.9
WAIT_TIME = 15


class TestInviteTeacher(TestCase):
    def test_invite_teacher_successful(self):
        email, password = signup_teacher_directly()
        school = create_organisation_directly(email)
        create_class_directly(email)
        teacher = Teacher.objects.get(new_user__email=email)

        client = Client()
        client.login(username=email, password=password)

        invited_teacher_first_name = "Valid"
        invited_teacher_last_name = "Name"
        invited_teacher_email = "valid_email@example.com"
        invited_teacher_password = "$RRFVBGT%^yhnmju7"

        # Invite another teacher to school and check they got an email
        dashboard_url = reverse("dashboard")
        data = {
            "teacher_first_name": invited_teacher_first_name,
            "teacher_last_name": invited_teacher_last_name,
            "teacher_email": invited_teacher_email,
            "invite_teacher": "",
        }
        assert len(mail.outbox) == 0
        response = client.post(dashboard_url, data)
        assert response.status_code == 200
        messages = list(response.context["messages"])
        assert len(messages) == 1
        assert (
            str(messages[0])
            == f"You have invited {invited_teacher_first_name} {invited_teacher_last_name} to your school."
        )
        assert len(mail.outbox) == 1
        client.logout()

        # Complete the registration as the invited teacher
        invitation = SchoolTeacherInvitation.objects.get(invited_teacher_email=invited_teacher_email)
        invitation_url = reverse("invited_teacher", kwargs={"token": invitation.token})
        response = client.post(
            invitation_url,
            {
                "teacher_signup-teacher_password": invited_teacher_password,
                "teacher_signup-teacher_confirm_password": invited_teacher_password,
                "teacher_signup-consent_ticked": "on",
            },
        )

        # Check the message displays correctly after registration
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert messages[0] == "Your account has been created successfully, please log in."

        # Check that the teacher account is created successfully and linked to the school
        invited_teacher = Teacher.objects.get(new_user__email=invited_teacher_email)
        assert invited_teacher.new_user.first_name == invited_teacher_first_name
        assert invited_teacher.new_user.last_name == invited_teacher_last_name
        assert invited_teacher.school == school
        assert invited_teacher.invited_by == teacher

        # Check that the invitation is now inactive
        with pytest.raises(SchoolTeacherInvitation.DoesNotExist):
            SchoolTeacherInvitation.objects.get(invited_teacher_email=invited_teacher_email)
        old_invitation = SchoolTeacherInvitation._base_manager.get(id=invitation.id)
        assert old_invitation.invited_teacher_first_name != invited_teacher_first_name
        assert old_invitation.invited_teacher_last_name != invited_teacher_last_name
        assert old_invitation.invited_teacher_email != invited_teacher_email
        assert not old_invitation.is_active

    def test_invite_teacher_fail(self):
        email, password = signup_teacher_directly()
        school = create_organisation_directly(email)
        create_class_directly(email)
        teacher = Teacher.objects.get(new_user__email=email)

        client = Client()
        client.login(username=email, password=password)

        # Try to invite a teacher with an invalid email address
        dashboard_url = reverse("dashboard")
        data = {
            "teacher_first_name": "Valid",
            "teacher_last_name": "Name",
            "teacher_email": "invalid_email",
            "invite_teacher": "",
        }
        response = client.post(dashboard_url, data)
        assert len(response.context["invite_teacher_form"]["teacher_email"].errors) == 1
        assert response.context["invite_teacher_form"]["teacher_email"].errors[0] == "Enter a valid email address."
        client.logout()

        # Try to access an invitation with an invalid token
        invitation_url = reverse("invited_teacher", kwargs={"token": "1"})
        response = client.get(invitation_url)
        assert response.context["error_message"] == "Uh oh, the Invitation does not exist or it has expired. 😞"

        # Try to access an expired invitation
        expired_invitation = SchoolTeacherInvitation.objects.create(
            token=uuid4().hex,
            school=school,
            from_teacher=teacher,
            invited_teacher_first_name="Valid",
            invited_teacher_last_name="Name",
            invited_teacher_email="valid@cfl.com",
            expiry=timezone.now() - timedelta(days=1),
        )
        invitation_url = reverse("invited_teacher", kwargs={"token": expired_invitation.token})
        response = client.get(invitation_url)
        assert response.context["error_message"] == "Uh oh, the Invitation does not exist or it has expired. 😞"

        # Try to access an invitation for an account that already exists
        same_account_invitation = SchoolTeacherInvitation.objects.create(
            token=uuid4().hex,
            school=school,
            from_teacher=teacher,
            invited_teacher_first_name="Valid",
            invited_teacher_last_name="Name",
            invited_teacher_email=email,
            expiry=timezone.now() + timedelta(days=1),
        )
        invitation_url = reverse("invited_teacher", kwargs={"token": same_account_invitation.token})
        response = client.get(invitation_url)
        assert response.context["error_message"] == (
            "It looks like an account is already registered with this email address. You will need to delete the "
            "other account first or change the email associated with it in order to proceed. You will then be able to "
            "access this page."
        )


class TestTeacherInviteAPI(TestCase):
    def test_delete_exception(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        create_class_directly(email)

        client = Client()
        client.login(username=email, password=password)

        response = client.post(reverse("delete_teacher_invite", kwargs={"token": "2345678"}))
        message = list(response.wsgi_request._messages)[0].message
        assert message == "You do not have permission to perform this action or the invite does not exist"

        response = client.post(reverse("resend_invite_teacher", kwargs={"token": "2345678"}))
        message = list(response.wsgi_request._messages)[0].message
        assert message == "You do not have permission to perform this action or the invite does not exist"


class TestTeacherInviteActions(BaseTest):
    def test_revoke_and_make_admin_invite(self):
        teacher_email, teacher_password = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        class_name = "Test Class"
        klass, _, _ = create_class_directly(teacher_email, class_name)

        page = self.go_to_homepage()
        page = page.go_to_teacher_login_page().login(teacher_email, teacher_password)

        # Generate an invite and make admin
        invite_data = {"teacher_first_name": "Adam", "teacher_last_name": "NotAdam", "teacher_email": "adam@adam.not"}
        for key in invite_data.keys():
            field = page.browser.find_element(By.NAME, key)
            field.send_keys(invite_data[key])

        # check if invite text for a user has been generated
        click_buttons_by_id(page, self, "invite_teacher_button")
        banner = page.browser.find_element(By.ID, "messages")
        assert (
            f"You have invited {invite_data['teacher_first_name']} {invite_data['teacher_last_name']} to your school."
            in banner.text
        )

        # check if popup message appears and if the invite is changed to admin
        sleep(1)  # this HAS to be there because of the animation :/
        click_buttons_by_id(page, self, ["make_admin_button_invite", "add_admin_button"])
        invite = SchoolTeacherInvitation.objects.filter(invited_teacher_first_name="Adam")[0]
        assert invite.invited_teacher_is_admin
        banner = page.browser.find_element(By.ID, "messages")
        assert "Administrator invite status has been given successfully" in banner.text

        # revoke admin
        click_buttons_by_id(page, self, "make_non_admin_button_invite")

        banner = page.browser.find_element(By.ID, "messages")
        assert "Administrator invite status has been revoked successfully" in banner.text

    def test_delete_invite(self):
        teacher_email, teacher_password = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        class_name = "Test Class"
        klass, _, _ = create_class_directly(teacher_email, class_name)

        page = self.go_to_homepage()
        page = page.go_to_teacher_login_page().login(teacher_email, teacher_password)

        # Generate an invite
        invite_data = {"teacher_first_name": "Adam", "teacher_last_name": "NotAdam", "teacher_email": "adam@adam.not"}
        for key in invite_data.keys():
            field = page.browser.find_element(By.NAME, key)
            field.send_keys(invite_data[key])
        invite_button = page.browser.find_element(By.NAME, "invite_teacher_button")
        invite_button.click()

        # check object was created
        invite_queryset = SchoolTeacherInvitation.objects.filter(invited_teacher_first_name="Adam")
        assert len(invite_queryset) == 1
        sleep(FADE_TIME)
        # delete
        delete_invite_button = WebDriverWait(self.selenium, WAIT_TIME).until(
            EC.element_to_be_clickable((By.ID, "delete-invite"))
        )
        delete_invite_button.click()

        empty_invite_queryset = SchoolTeacherInvitation.objects.filter(invited_teacher_first_name="Adam")
        assert len(empty_invite_queryset) == 0

    def test_resend_invite(self):
        teacher_email, teacher_password = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        class_name = "Test Class"
        klass, _, _ = create_class_directly(teacher_email, class_name)

        page = self.go_to_homepage()
        page = page.go_to_teacher_login_page().login(teacher_email, teacher_password)

        # Generate an invite
        invite_data = {"teacher_first_name": "Adam", "teacher_last_name": "NotAdam", "teacher_email": "adam@adam.not"}
        for key in invite_data.keys():
            field = page.browser.find_element(By.NAME, key)
            field.send_keys(invite_data[key])

        click_buttons_by_id(page, self, "invite_teacher_button")

        banner = page.browser.find_element(By.XPATH, '//*[@id="messages"]/div/div/div/div/div/p')
        assert (
            banner.text
            == f"You have invited {invite_data['teacher_first_name']} {invite_data['teacher_last_name']} to your school."
        )

        # resend an invite
        click_buttons_by_id(page, self, "resend-invite")

        # check if invite was updated by 30 days (used 29 for rounding errors)
        new_invite_expiry = SchoolTeacherInvitation.objects.filter(invited_teacher_first_name="Adam")[0].expiry
        assert timezone.now() + timedelta(days=29) <= new_invite_expiry



from __future__ import unicode_literals

from builtins import str
from django.db import models, migrations


class Migration(migrations.Migration):

    # It will guess the country of the school using postcode, UK will be set if no result is returned by the API
    def populate_country(apps, schema_editor):

        School = apps.get_model("portal", "School")
        for school in School.objects.all():
            country, town, lat, lng = (
                "GB",
                "0",
                "0",
                "0",
            )
            school.country = str(country)
            school.town = town
            school.lat = lat
            school.lng = lng
            school.save()

    def reset_country(apps, schema_editor):
        School = apps.get_model("portal", "School")
        for school in School.objects.all():
            school.country = ""
            school.town = "0"
            school.lat = "0"
            school.lng = "0"
            school.save()

    dependencies = [("portal", "0043_auto_20150430_0952")]

    operations = [migrations.RunPython(populate_country, reset_country)]



from __future__ import absolute_import

import time
from datetime import timedelta
from unittest.mock import ANY, Mock, patch
from uuid import uuid4

import jwt
from common.mail import campaign_ids
from common.tests.utils import email as email_utils
from common.tests.utils.classes import create_class_directly
from common.tests.utils.organisation import (
    create_organisation_directly,
    join_teacher_to_organisation,
)
from common.tests.utils.student import (
    create_independent_student_directly,
    create_school_student_directly,
)
from common.tests.utils.teacher import (
    signup_duplicate_teacher_fail,
    signup_teacher,
    signup_teacher_directly,
    verify_email,
)
from django.conf import settings
from django.core import mail
from django.test import Client, TestCase
from django.urls import reverse
from django.utils import timezone
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from portal.forms.error_messages import INVALID_LOGIN_MESSAGE
from portal.tests.base_test import click_buttons_by_id
from portal.tests.test_invite_teacher import WAIT_TIME

from .base_test import BaseTest
from .pageObjects.portal.home_page import HomePage
from .utils.messages import (
    is_email_updated_message_showing,
    is_email_verified_message_showing,
    is_message_showing,
    is_password_updated_message_showing,
    is_teacher_details_updated_message_showing,
)


class TestTeacher(TestCase):
    def test_signup_short_password_fails(self):
        c = Client()

        response = c.post(
            reverse("register"),
            {
                "teacher_signup-teacher_first_name": "Test Name",
                "teacher_signup-teacher_last_name": "Test Last Name",
                "teacher_signup-teacher_email": "test@email.com",
                "teacher_signup-consent_ticked": "on",
                "teacher_signup-teacher_password": "test",
                "teacher_signup-teacher_confirm_password": "test",
                "g-recaptcha-response": "something",
            },
        )

        # Assert response isn't a redirect (submit failure)
        assert response.status_code == 200

    def test_signup_common_password_fails(self):
        c = Client()

        response = c.post(
            reverse("register"),
            {
                "teacher_signup-teacher_first_name": "Test Name",
                "teacher_signup-teacher_last_name": "Test Last Name",
                "teacher_signup-teacher_email": "test@email.com",
                "teacher_signup-consent_ticked": "on",
                "teacher_signup-teacher_password": "Password1",
                "teacher_signup-teacher_confirm_password": "Password1",
                "g-recaptcha-response": "something",
            },
        )

        # Assert response isn't a redirect (submit failure)
        assert response.status_code == 200

        response = c.post(
            reverse("register"),
            {
                "independent_student_signup-date_of_birth_day": 7,
                "independent_student_signup-date_of_birth_month": 10,
                "independent_student_signup-date_of_birth_year": 1997,
                "independent_student_signup-name": "Test Name",
                "independent_student_signup-email": "test@email.com",
                "independent_student_signup-consent_ticked": "on",
                "independent_student_signup-password": "Password123$",
                "independent_student_signup-confirm_password": "Password123$",
                "g-recaptcha-response": "something",
            },
        )
        assert response.status_code == 200

    def test_signup_passwords_do_not_match_fails(self):
        c = Client()

        response = c.post(
            reverse("register"),
            {
                "teacher_signup-teacher_first_name": "Test Name",
                "teacher_signup-teacher_last_name": "Test Last Name",
                "teacher_signup-teacher_email": "test@email.com",
                "teacher_signup-consent_ticked": "on",
                "teacher_signup-teacher_password": "StrongPassword1!",
                "teacher_signup-teacher_confirm_password": "StrongPassword2!",
                "g-recaptcha-response": "something",
            },
        )

        # Assert response isn't a redirect (submit failure)
        assert response.status_code == 200

    def test_signup_fails_without_consent(self):
        c = Client()

        response = c.post(
            reverse("register"),
            {
                "teacher_signup-teacher_first_name": "Test Name",
                "teacher_signup-teacher_last_name": "Test Last Name",
                "teacher_signup-teacher_email": "test@email.com",
                "teacher_signup-teacher_password": "StrongPassword1!",
                "teacher_signup-teacher_confirm_password": "StrongPassword1!",
                "g-recaptcha-response": "something",
            },
        )

        # Assert response isn't a redirect (submit failure)
        assert response.status_code == 200

    @patch("common.helpers.emails.send_dotdigital_email")
    def test_signup_email_verification(self, mock_send_dotdigital_email: Mock):
        c = Client()

        response = c.post(
            reverse("register"),
            {
                "teacher_signup-teacher_first_name": "Test Name",
                "teacher_signup-teacher_last_name": "Test Last Name",
                "teacher_signup-teacher_email": "test@email.com",
                "teacher_signup-consent_ticked": "on",
                "teacher_signup-teacher_password": "czYuH)g0FbD_5E9/",
                "teacher_signup-teacher_confirm_password": "czYuH)g0FbD_5E9/",
                "g-recaptcha-response": "something",
            },
        )

        assert response.status_code == 302
        mock_send_dotdigital_email.assert_called_once_with(
            campaign_ids["verify_new_user"], ANY, personalization_values=ANY
        )

        # Try verification URL with a fake token
        fake_token = jwt.encode(
            {
                "email": "fake_email",
                "new_email": "",
                "email_verification_token": uuid4().hex[:30],
                "expires": (timezone.now() + timedelta(hours=1)).timestamp(),
            },
            settings.SECRET_KEY,
            algorithm="HS256",
        )
        bad_url = reverse("verify_email", kwargs={"token": fake_token})
        bad_verification_response = c.get(bad_url)

        # Assert response isn't a redirect (get failure)
        assert bad_verification_response.status_code == 200

        # Get verification link from function call
        verification_url = mock_send_dotdigital_email.call_args.kwargs[
            "personalization_values"
        ]["VERIFICATION_LINK"]

        # Verify the email properly
        verification_response = c.get(verification_url)

        # Assert response redirects and succeeds
        assert verification_response.status_code == 302

        # Try verifying the email a second time
        second_verification_response = c.get(verification_url)

        # Assert response isn't a redirect (get failure)
        assert second_verification_response.status_code == 200


# Class for Selenium tests. We plan to replace these and turn them into Cypress tests
class TestTeacherFrontend(BaseTest):
    def test_password_too_common(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_signup_page()
        page = page.signup(
            "first_name",
            "last_name",
            "e@ma.il",
            "Password123$",
            "Password123$",
            success=False,
        )
        try:
            submit_button = WebDriverWait(self.selenium, 10).until(
                EC.element_to_be_clickable((By.NAME, "teacher_signup"))
            )
            submit_button.click()
        except:
            assert page.was_form_invalid(
                "form-reg-teacher",
                "Password is too common, consider using a different password.",
            )

    def test_signup_without_newsletter(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page, _, _ = signup_teacher(page)
        assert is_email_verified_message_showing(self.selenium)

    def test_signup_with_newsletter(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page, _, _ = signup_teacher(page, newsletter=True)
        assert is_email_verified_message_showing(self.selenium)

    def test_signup_duplicate_failure(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page, email, _ = signup_teacher(page)
        assert is_email_verified_message_showing(self.selenium)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page, _, _ = signup_duplicate_teacher_fail(page, email)
        assert self.is_login_page(page)

        # Test sign up with an existing indy student's email
        indy_email, _, _ = create_independent_student_directly()
        page = self.go_to_homepage()
        page, _, _ = signup_duplicate_teacher_fail(page, indy_email)

    def test_login_failure(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page = page.go_to_teacher_login_page()
        page = page.login_failure(
            "non-existent-email@codeforlife.com", "Incorrect password"
        )
        assert page.has_login_failed(
            "form-login-teacher", INVALID_LOGIN_MESSAGE
        )

    def test_login_success(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page = page.go_to_teacher_login_page()
        page = page.login(email, password)
        assert self.is_dashboard_page(page)

    @patch("common.helpers.emails.send_dotdigital_email")
    def test_login_not_verified(self, mock_send_dotdigital_email):
        email, password = signup_teacher_directly(preverified=False)
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page = page.go_to_teacher_login_page()
        page = page.login_failure(email, password)

        assert page.has_login_failed(
            "form-login-teacher", INVALID_LOGIN_MESSAGE
        )

        verification_url = mock_send_dotdigital_email.call_args.kwargs[
            "personalization_values"
        ]["VERIFICATION_LINK"]

        verify_email(page, verification_url)

        assert is_email_verified_message_showing(self.selenium)

        page = page.login(email, password)

        assert self.is_dashboard_page(page)

    def test_signup_login_success(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page, email, password = signup_teacher(page)
        page = page.login_no_school(email, password)
        assert self.is_onboarding_page(page)

    def test_edit_details(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, password)
            .open_account_tab()
        )

        page = page.change_teacher_details(
            {
                "first_name": "Paulina",
                "last_name": "Koch",
                "current_password": "$RFVBGT%6yhn$RFVBGT%6yhn$RFVBGT%6yhn$RFVBGT%6yhn",
            }
        )
        assert self.is_dashboard_page(page)
        assert is_teacher_details_updated_message_showing(self.selenium)

        assert page.check_account_details(
            {"first_name": "Paulina", "last_name": "Koch"}
        )

    def test_edit_details_non_admin(self):
        email_1, _ = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        school = create_organisation_directly(email_1)
        _, _, access_code_1 = create_class_directly(email_1)
        create_school_student_directly(access_code_1)
        join_teacher_to_organisation(email_2, school.name)
        _, _, access_code_2 = create_class_directly(email_2)
        create_school_student_directly(access_code_2)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email_2, password_2)
            .open_account_tab()
        )

        page = page.change_teacher_details(
            {
                "first_name": "Florian",
                "last_name": "Aucomte",
                "current_password": password_2,
            }
        )
        assert self.is_dashboard_page(page)
        assert is_teacher_details_updated_message_showing(self.selenium)

        assert page.check_account_details(
            {"first_name": "Florian", "last_name": "Aucomte"}
        )

    @patch("common.helpers.emails.send_dotdigital_email")
    def test_change_email(self, mock_send_dotdigital_email):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        other_email, _ = signup_teacher_directly()

        page = self.go_to_homepage()
        page = (
            page.go_to_teacher_login_page()
            .login(email, password)
            .open_account_tab()
        )

        # Try changing email to an existing email, should fail
        page = page.change_email("Test", "Teacher", other_email, password)
        assert self.is_email_verification_page(page)
        assert is_email_updated_message_showing(self.selenium)

        mock_send_dotdigital_email.assert_called_with(
            campaign_ids["email_change_notification"],
            ANY,
            personalization_values=ANY,
        )

        # Try changing email to an existing indy student's email, should fail
        indy_email, _, _ = create_independent_student_directly()
        page = self.go_to_homepage()
        page = (
            page.go_to_teacher_login_page()
            .login(email, password)
            .open_account_tab()
        )

        page = page.change_email("Test", "Teacher", indy_email, password)
        assert self.is_email_verification_page(page)
        assert is_email_updated_message_showing(self.selenium)

        mock_send_dotdigital_email.assert_called_with(
            campaign_ids["email_change_notification"],
            ANY,
            personalization_values=ANY,
        )

        page = self.go_to_homepage()
        page = (
            page.go_to_teacher_login_page()
            .login(email, password)
            .open_account_tab()
        )

        # Try changing email to a new one, should succeed
        new_email = "another-email@codeforlife.com"
        page = page.change_email("Test", "Teacher", new_email, password)
        assert self.is_email_verification_page(page)
        assert is_email_updated_message_showing(self.selenium)

        # Check user can still log in with old account before verifying new email
        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, password)
        )
        assert self.is_dashboard_page(page)

        page = page.logout()

        mock_send_dotdigital_email.assert_called_with(
            campaign_ids["email_change_verification"],
            ANY,
            personalization_values=ANY,
        )
        verification_url = mock_send_dotdigital_email.call_args.kwargs[
            "personalization_values"
        ]["VERIFICATION_LINK"]

        page = email_utils.follow_change_email_link_to_dashboard(
            page, verification_url
        )

        page = page.login(new_email, password).open_account_tab()

        assert page.check_account_details(
            {"first_name": "Test", "last_name": "Teacher"}
        )

    def test_change_password(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, password)
            .open_account_tab()
        )

        new_password = "AnotherPassword12!"
        page = page.change_password("Test", "Teacher", new_password, password)
        assert self.is_login_page(page)
        assert is_password_updated_message_showing(self.selenium)

        page = page.login(email, new_password)

        assert self.is_dashboard_page(page)

    @patch("portal.forms.registration.send_dotdigital_email")
    def test_reset_password(self, mock_send_dotdigital_email: Mock):
        email, _ = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        page = self.get_to_forgotten_password_page()

        page.reset_email_submit(email)

        mock_send_dotdigital_email.assert_called_with(
            campaign_ids["reset_password"], ANY, personalization_values=ANY
        )

        reset_password_url = mock_send_dotdigital_email.call_args.kwargs[
            "personalization_values"
        ]["RESET_PASSWORD_LINK"]

        page = email_utils.follow_reset_email_link(
            self.selenium, reset_password_url
        )

        new_password = "AnotherPassword12!"

        page.teacher_reset_password(new_password)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, new_password)
        )
        assert self.is_dashboard_page(page)

    @patch("portal.forms.registration.send_dotdigital_email")
    def test_reset_with_same_password(self, mock_send_dotdigital_email: Mock):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        page = self.get_to_forgotten_password_page()

        page.reset_email_submit(email)

        mock_send_dotdigital_email.assert_called_with(
            campaign_ids["reset_password"], ANY, personalization_values=ANY
        )

        reset_password_url = mock_send_dotdigital_email.call_args.kwargs[
            "personalization_values"
        ]["RESET_PASSWORD_LINK"]

        page = email_utils.follow_reset_email_link(
            self.selenium, reset_password_url
        )

        page.reset_password_fail(password)

        message = page.browser.find_element(By.CLASS_NAME, "errorlist")
        assert (
            "Please choose a password that you haven't used before"
            in message.text
        )

    @patch("portal.forms.registration.send_dotdigital_email")
    def test_reset_password_fail(self, mock_send_dotdigital_email: Mock):
        page = self.get_to_forgotten_password_page()
        fake_email = "fake_email@fakeemail.com"
        page.reset_email_submit(fake_email)

        mock_send_dotdigital_email.assert_not_called()

    def test_admin_sees_all_school_classes(self):
        email, password = signup_teacher_directly()
        school = create_organisation_directly(email)
        klass, _, access_code = create_class_directly(email, "class123")

        # create non_admin account to join the school
        # check if they cannot see classes
        standard_email, standard_password = signup_teacher_directly()
        join_teacher_to_organisation(standard_email, school.name)

        page = (
            self.go_to_homepage()
            .go_to_teacher_login_page()
            .login(standard_email, standard_password)
            .open_classes_tab()
        )

        assert page.element_does_not_exist_by_id(f"class-code-{access_code}")

        self.go_to_homepage().teacher_logout()
        # then make an admin account and check
        # if the teacher can see the classes

        admin_email, admin_password = signup_teacher_directly()
        join_teacher_to_organisation(admin_email, school.name, is_admin=True)

        page = (
            self.go_to_homepage()
            .go_to_teacher_login_page()
            .login(admin_email, admin_password)
            .open_classes_tab()
        )
        class_code_field = page.browser.find_element(
            By.ID, f"class-code-{access_code}"
        )
        assert class_code_field.text == access_code

    def test_admin_student_edit(self):
        email, password = signup_teacher_directly()
        school = create_organisation_directly(email)

        klass, _, access_code = create_class_directly(email, "class123")
        (
            student_name,
            student_password,
            student_student,
        ) = create_school_student_directly(access_code)

        joining_email, joining_password = signup_teacher_directly()
        join_teacher_to_organisation(joining_email, school.name, is_admin=True)

        page = (
            self.go_to_homepage()
            .go_to_teacher_login_page()
            .login(joining_email, joining_password)
            .open_classes_tab()
        )

        class_button = WebDriverWait(self.selenium, WAIT_TIME).until(
            EC.element_to_be_clickable((By.ID, "class_button"))
        )
        class_button.click()

        edit_student_button = WebDriverWait(self.selenium, WAIT_TIME).until(
            EC.element_to_be_clickable((By.ID, "edit_student_button"))
        )
        edit_student_button.click()

        title = page.browser.find_element(By.ID, "student_details")
        assert (
            title.text
            == f"Edit student details for {student_name} from class {klass} ({access_code})"
        )

    def test_make_admin_popup(self):
        email, password = signup_teacher_directly()
        school = create_organisation_directly(email)
        page = (
            self.go_to_homepage()
            .go_to_teacher_login_page()
            .login(email, password)
        )
        joining_email, _ = signup_teacher_directly()

        invite_data = {
            "teacher_first_name": "Real",
            "teacher_last_name": "Name",
            "teacher_email": "ren@me.me",
        }

        for key in invite_data.keys():
            field = page.browser.find_element(By.NAME, key)
            field.send_keys(invite_data[key])

        invite_button = page.browser.find_element(
            By.NAME, "invite_teacher_button"
        )
        invite_button.click()

        # Once invite sent test the make admin button
        """
        make_admin_button = WebDriverWait(self.selenium, WAIT_TIME).until(
            EC.element_to_be_clickable((By.ID, "make_admin_button_invite"))
        )
        make_admin_button.click()
        """

        button_ids = [
            "make_admin_button_invite",
            "cancel_admin_popup_button",
            "delete-invite",
        ]
        click_buttons_by_id(page, self, button_ids)

        # Delete the invite and check if the form invite with
        # admin checked also makes a popup

        for key in invite_data.keys():
            field = page.browser.find_element(By.NAME, key)
            field.send_keys(invite_data[key])
        checkbox = page.browser.find_element(By.NAME, "make_admin_ticked")
        checkbox.click()

        button_ids = ["invite_teacher_button", "cancel_admin_popup_button"]

        click_buttons_by_id(page, self, button_ids)

        # Non admin teacher joined - make admin should also make a popup

        join_teacher_to_organisation(joining_email, school.name)

        # refresh the page and scroll to the buttons
        page.browser.execute_script("location.reload()")
        click_buttons_by_id(page, self, "make_admin_button")

        assert page.element_exists((By.CLASS_NAME, "popup-box__msg"))

    def test_delete_account(self):
        FADE_TIME = 0.9  # often fails if lower

        email, password = signup_teacher_directly()
        create_organisation_directly(email)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, password)
            .open_account_tab()
        )

        # test incorrect password
        page.browser.find_element(By.ID, "id_delete_password").send_keys(
            "IncorrectPassword"
        )
        page.browser.find_element(By.ID, "delete_account_button").click()
        is_message_showing(page.browser, "Your account was not deleted")

        # test cancel (no class)
        time.sleep(FADE_TIME)
        page.browser.find_element(By.ID, "id_delete_password").clear()
        page.browser.find_element(By.ID, "id_delete_password").send_keys(
            password
        )
        page.browser.find_element(By.ID, "delete_account_button").click()

        time.sleep(FADE_TIME)
        assert page.browser.find_element(
            By.ID, "popup-delete-review"
        ).is_displayed()
        page.browser.find_element(By.ID, "cancel_popup_button").click()
        time.sleep(FADE_TIME)

        # test close button in the corner
        page.browser.find_element(By.ID, "id_delete_password").clear()
        page.browser.find_element(By.ID, "id_delete_password").send_keys(
            password
        )
        page.browser.find_element(By.ID, "delete_account_button").click()

        time.sleep(FADE_TIME)
        page.browser.find_element(By.ID, "close_popup_button").click()
        time.sleep(FADE_TIME)

        # create class
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        # delete then review classes
        page.browser.find_element(By.ID, "id_delete_password").send_keys(
            password
        )
        page.browser.find_element(By.ID, "delete_account_button").click()

        time.sleep(FADE_TIME)
        assert page.browser.find_element(
            By.ID, "popup-delete-review"
        ).is_displayed()
        page.browser.find_element(By.ID, "review_button").click()
        time.sleep(FADE_TIME)

        assert page.has_classes()
        page = page.open_account_tab()

        # test actual deletion
        page.browser.find_element(By.ID, "id_delete_password").send_keys(
            password
        )
        page.browser.find_element(By.ID, "delete_account_button").click()

        time.sleep(FADE_TIME)
        page.browser.find_element(By.ID, "delete_button").click()

        # back to homepage
        assert page.browser.find_element(By.CLASS_NAME, "banner--homepage")

        # user should not be able to login now
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_failure(email, password)
        )

        assert page.has_login_failed(
            "form-login-teacher", INVALID_LOGIN_MESSAGE
        )

    def test_onboarding_complete(self):
        email, password = signup_teacher_directly()

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_school(email, password)
        )

        page = page.create_organisation("Test school", "W1", "GB")
        page = page.create_class("Test class", True)
        page = (
            page.type_student_name("Test Student")
            .create_students()
            .complete_setup()
        )

        assert page.has_onboarding_complete_popup()

    def get_to_forgotten_password_page(self):
        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .go_to_teacher_forgotten_password_page()
        )
        return page

    def wait_for_email(self):
        WebDriverWait(self.selenium, 2).until(
            lambda driver: len(mail.outbox) == 1
        )

    def is_dashboard_page(self, page):
        return page.__class__.__name__ == "TeachDashboardPage"

    def is_resources_page(self, page):
        return page.__class__.__name__ == "ResourcesPage"

    def is_onboarding_page(self, page):
        return page.__class__.__name__ == "OnboardingOrganisationPage"

    def is_login_page(self, page):
        return page.__class__.__name__ == "TeacherLoginPage"

    def is_email_verification_page(self, page):
        return page.__class__.__name__ == "EmailVerificationNeededPage"



from collections import namedtuple

import pytest
from common.models import Class
from common.tests.utils.classes import create_class_directly
from common.tests.utils.organisation import create_organisation_directly
from common.tests.utils.student import (
    create_independent_student_directly,
    create_school_student_directly,
)
from common.tests.utils.teacher import signup_teacher_directly

SchoolStudent = namedtuple("student", ["username", "password"])
IndependentStudent = namedtuple("independent_student", ["username", "password"])
TeacherLoginDetails = namedtuple("teacher", ["email", "password"])


@pytest.fixture
def teacher1(db) -> TeacherLoginDetails:
    return TeacherLoginDetails(*signup_teacher_directly())


@pytest.fixture
def class1(db, teacher1: TeacherLoginDetails) -> Class:
    create_organisation_directly(teacher1.email)
    klass, _, _ = create_class_directly(teacher1.email)
    return klass


@pytest.fixture
def student1(db, class1) -> SchoolStudent:
    username, password, _ = create_school_student_directly(class1.access_code)
    return SchoolStudent(username, password)


@pytest.fixture
def independent_student1(db) -> IndependentStudent:
    username, password, _ = create_independent_student_directly()
    return IndependentStudent(username, password)



from django.urls import reverse
from django.test import TestCase, Client


class TestGlobalForms(TestCase):
    def test_newsletter_signup_successful(self):
        url = reverse("process_newsletter_form")
        client = Client()
        data = {"email": "valid_email@example.com", "age_verification": "on"}
        response = client.post(url, data)
        messages = list(response.wsgi_request._messages)
        assert len([m for m in messages if m.tags == "success"]) == 1

    def test_newsletter_signup_fail(self):
        url = reverse("process_newsletter_form")
        client = Client()
        data = {"email": "invalid_email", "age_verification": "on"}
        response = client.post(url, data)
        messages = list(response.wsgi_request._messages)
        assert len([m for m in messages if "error" in m.tags]) == 1



from __future__ import unicode_literals

from uuid import uuid4
from datetime import timedelta

from django.db import migrations
from django.utils import timezone


def add_missing_ev_records(apps, schema_editor):
    User = apps.get_model("auth", "User")
    EmailVerification = apps.get_model("portal", "EmailVerification")
    verified_users_without_ev = User.objects.filter(
        userprofile__awaiting_email_verification=False
    ).exclude(email_verifications__verified=True)

    records = [
        EmailVerification(
            user=user,
            email=user.email,
            token=uuid4().hex[:30],
            expiry=timezone.now() + timedelta(hours=1),
            verified=True,
        )
        for user in verified_users_without_ev
    ]

    EmailVerification.objects.bulk_create(records)


def reverse(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [("portal", "0050_refactor_emailverifications_2")]

    operations = [migrations.RunPython(add_missing_ev_records, reverse)]



import json

import requests
from django.http import HttpRequest, JsonResponse
from django.views.decorators.csrf import csrf_exempt


@csrf_exempt
def collect(request: HttpRequest):
    body: dict = json.loads(request.body)
    measurement_id = body["measurement_id"]
    api_secret = body["api_secret"]
    debug = body.get("debug", False)

    response = requests.post(
        url=(
            "https://www.google-analytics.com/debug/mp/collect"
            if debug
            else "https://www.google-analytics.com/mp/collect"
        )
        + f"?measurement_id=${measurement_id}&api_secret=${api_secret}",
        json=body["payload"],
    )

    return JsonResponse(
        data=response.json() if debug else {},
        status=response.status_code,
    )



PLAY_BANNER = {
    "title": "Play",
    "subtitle": "Anyone can learn how to code. We will help you learn how. It’s " "fun, free and easy.",
    "text": "",
    "button": {"text": "Try Rapid Router", "link": "levels"},
    "button2": {"text": "Try Python Den", "link": "python_levels"},
    "image_class": "banner--picture--play",
    "alt": "Boy playing on ipad",
    "image_description": "Credit: Annie Spratt, Unsplash",
}

PLAY_HEADLINE = {
    "title": "Anyone can code, you can too!",
    "description": "Whether you’re a parent, teacher or a student, our games support "
    "and guide you, making learning to code great fun. Get started with Rapid Router "
    "designed for students new to coding. Rapid Router is where you will build up your "
    "ability.",
}



from __future__ import absolute_import

import time

from common.models import Teacher
from common.tests.utils.classes import create_class_directly
from common.tests.utils.organisation import (
    create_organisation,
    create_organisation_directly,
    join_teacher_to_organisation,
)
from common.tests.utils.student import create_school_student_directly
from common.tests.utils.teacher import signup_teacher_directly
from selenium.webdriver.common.by import By

from portal.tests.pageObjects.portal.base_page import BasePage
from portal.tests.pageObjects.portal.home_page import HomePage
from portal.tests.test_invite_teacher import FADE_TIME

from .base_test import BaseTest
from .utils.messages import is_organisation_created_message_showing


class TestOrganisation(BaseTest, BasePage):
    def test_create(self):
        email, password = signup_teacher_directly()

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login_no_school(email, password)

        page, name = create_organisation(page, password)
        assert is_organisation_created_message_showing(self.selenium, name)

    def test_kick(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        school = create_organisation_directly(email_1)
        _, _, access_code = create_class_directly(email_1)
        create_school_student_directly(access_code)

        join_teacher_to_organisation(email_2, school.name)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email_1, password_1)

        teacher2 = Teacher.objects.get(new_user__email=email_2)

        assert page.is_teacher_in_school(teacher2.new_user.last_name)

        page = page.click_kick_button()
        assert page.is_dialog_showing()
        page = page.confirm_dialog()

        assert page.is_not_teacher_in_school(teacher2.new_user.last_name)

    def test_move_classes_and_kick(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        school = create_organisation_directly(email_1)
        _, _, access_code_1 = create_class_directly(email_1)
        create_school_student_directly(access_code_1)
        _, _, access_code_2 = create_class_directly(email_2)
        create_school_student_directly(access_code_2)

        join_teacher_to_organisation(email_2, school.name)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email_1, password_1)

        teacher2 = Teacher.objects.get(new_user__email=email_2)

        assert page.is_teacher_in_school(teacher2.new_user.last_name)

        page = page.click_kick_button()
        assert page.is_dialog_showing()
        page = page.confirm_kick_with_students_dialog()

        assert page.__class__.__name__ == "TeachMoveClassesPage"

        page = page.move_and_kick()

        assert page.is_not_teacher_in_school(teacher2.new_user.last_name)

    def test_leave_organisation(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        school = create_organisation_directly(email_1)
        _, _, access_code_1 = create_class_directly(email_1)
        create_school_student_directly(access_code_1)
        _, class_name_2, access_code_2 = create_class_directly(email_2)
        create_school_student_directly(access_code_2)

        join_teacher_to_organisation(email_2, school.name)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email_2, password_2)

        page = page.leave_organisation_with_students()

        assert page.__class__.__name__ == "TeachMoveClassesPage"

        page = page.move_and_leave()

        assert page.__class__.__name__ == "OnboardingOrganisationPage"

        page = page.logout().go_to_teacher_login_page().login(email_1, password_1)

        teacher2 = Teacher.objects.get(new_user__email=email_2)

        assert page.is_not_teacher_in_school(teacher2.new_user.last_name)

    def test_toggle_admin(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, _ = signup_teacher_directly()
        school = create_organisation_directly(email_1)
        _, _, access_code = create_class_directly(email_1)
        create_school_student_directly(access_code)
        join_teacher_to_organisation(email_2, school.name)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email_1, password_1)

        assert page.__class__.__name__ == "TeachDashboardPage"

        page = page.click_make_admin_button()
        time.sleep(FADE_TIME)
        popup_make_admin_button = page.browser.find_element(By.ID, "add_admin_button")
        assert popup_make_admin_button.text == "Add as admin"
        popup_make_admin_button.click()

        assert page.is_teacher_admin()

        page = page.click_make_non_admin_button()

        assert page.is_teacher_non_admin()

    def test_disable_2FA(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, _ = signup_teacher_directly()
        school = create_organisation_directly(email_1)
        _, _, access_code = create_class_directly(email_1)
        create_school_student_directly(access_code)
        join_teacher_to_organisation(email_2, school.name)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email_1, password_1)

        assert page.__class__.__name__ == "TeachDashboardPage"

        page = page.click_make_admin_button()
        # check if the new popup appears
        time.sleep(FADE_TIME)
        make_admin_button = page.browser.find_element(By.ID, "add_admin_button")
        assert make_admin_button.text == "Add as admin"
        make_admin_button.click()

        assert page.is_teacher_admin()

        page = page.click_make_non_admin_button()

        assert page.is_teacher_non_admin()

    def test_edit_details(self):
        email, password = signup_teacher_directly()
        school = create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email, password)

        assert page.check_organisation_details({"name": school.name})

        new_name = "new " + school.name

        page.change_organisation_details({"name": new_name})
        assert page.check_organisation_details({"name": new_name})

    def test_edit_clash(self):
        email_1, _ = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        school1 = create_organisation_directly(email_1)
        create_organisation_directly(email_2)
        _, _, access_code_1 = create_class_directly(email_1)
        _, _, access_code_2 = create_class_directly(email_2)
        create_school_student_directly(access_code_1)
        create_school_student_directly(access_code_2)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email_2, password_2)

        assert not page.check_organisation_details({"name": school1.name})

        page = page.change_organisation_details({"name": school1.name})

        assert page.has_edit_failed()



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [("portal", "0053_refactor_teacher_student_1")]

    operations = [
        migrations.AlterField(
            model_name="teacher",
            name="pending_join_request",
            field=models.ForeignKey(
                related_name="join_request",
                blank=True,
                to="portal.School",
                null=True,
                on_delete=models.SET_NULL,
            ),
        )
    ]



import logging


class ExceptionLoggingMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        return response

    def process_exception(self, request, exception):
        logging.exception(
            "Exception occurred while handling %s request to %s",
            request.method,
            request.path,
        )

        return None



from selenium.webdriver.common.by import By

from .base_page import BasePage
from .password_reset_page import PasswordResetPage
from .teach import dashboard_page as teach_dashboard_page


class TeacherLoginPage(BasePage):
    def __init__(self, browser):
        super(TeacherLoginPage, self).__init__(browser)

        assert self.on_correct_page("teacher_login_page")

    def login(self, email, password):
        self._login(email, password)
        return teach_dashboard_page.TeachDashboardPage(self.browser)

    def _login(self, email, password):
        self.browser.find_element(By.ID, "id_auth-username").send_keys(email)
        self.browser.find_element(By.ID, "id_auth-password").send_keys(password)
        self.browser.find_element(By.NAME, "login_view").click()

    def login_no_school(self, email, password):
        self._login(email, password)
        import portal.tests.pageObjects.portal.teach.onboarding_organisation_page as onboarding_organisation_page

        return onboarding_organisation_page.OnboardingOrganisationPage(self.browser)

    def login_no_class(self, email, password):
        self._login(email, password)

        return teach_dashboard_page.TeachDashboardPage(self.browser)

    def login_no_students(self, email, password):
        self._login(email, password)

        return teach_dashboard_page.TeachDashboardPage(self.browser)

    def login_failure(self, email, password):
        self._login(email, password)
        return self

    def has_login_failed(self, form_id, error):
        errors = self.browser.find_element(By.ID, form_id).find_element(By.CLASS_NAME, "errorlist").text
        return error in errors

    def go_to_teacher_forgotten_password_page(self):
        self.browser.find_element(By.ID, "teacher_forgotten_password_button").click()
        return PasswordResetPage(self.browser)



from django.shortcuts import render

from portal.strings.teach import TEACH_BANNER


def teach(request):
    return render(
        request,
        "portal/teach.html",
        {"BANNER": TEACH_BANNER},
    )



from portal.tests.pageObjects.portal.admin.admin_base_page import AdminBasePage


class AdminMapPage(AdminBasePage):
    def __init__(self, browser, live_server_url):
        super(AdminMapPage, self).__init__(browser, live_server_url)

        assert self.on_correct_page("admin_map")



from typing import Any, Dict, List, Optional

from common.mail import campaign_ids, send_dotdigital_email
from common.models import Student
from common.permissions import (
    logged_in_as_independent_student,
    logged_in_as_school_student,
)
from common.utils import LoginRequiredNoErrorMixin
from django.contrib import messages
from django.contrib.auth.decorators import login_required, user_passes_test
from django.contrib.auth.mixins import UserPassesTestMixin
from django.db.models.query import QuerySet
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse_lazy
from django.views.generic.base import TemplateView
from django.views.generic.edit import FormView
from game.models import Attempt, Level

from portal.forms.play import StudentJoinOrganisationForm


class SchoolStudentDashboard(
    LoginRequiredNoErrorMixin, UserPassesTestMixin, TemplateView
):
    template_name = "portal/play/student_dashboard.html"
    login_url = reverse_lazy("student_login_access_code")

    def test_func(self) -> Optional[bool]:
        return logged_in_as_school_student(self.request.user)

    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]:
        """
        Gathers the context data required by the template. First, the student's
        scores for the original Rapid Router levels is gathered, second,
        the student's scores for any levels shared with them by their teacher.
        """
        # Get score data for all original levels
        rapid_router_levels = Level.objects.filter(episode__pk__in=range(1, 10))
        python_den_levels = Level.objects.filter(
            episode__pk__in=[12, 13, 14, 15, 22]
        )
        student = self.request.user.new_student

        context_data = {
            "rapid_router": _compute_scores(student, rapid_router_levels),
            "python_den": _compute_scores(student, python_den_levels),
        }

        # Find any custom levels created by the teacher and shared with the
        # student
        klass = student.class_field
        teacher = klass.teacher.user
        custom_levels = student.new_user.shared.filter(owner=teacher)

        if custom_levels:
            custom_levels_data = _compute_scores(student, custom_levels)

            context_data["rapid_router"]["total_custom_score"] = (
                custom_levels_data
            )["total_score"]

            context_data["rapid_router"][
                "total_custom_available_score"
            ] = custom_levels_data["total_available_score"]

        return context_data


class IndependentStudentDashboard(
    LoginRequiredNoErrorMixin, UserPassesTestMixin, TemplateView, FormView
):
    template_name = "portal/play/student_dashboard.html"
    login_url = reverse_lazy("independent_student_login")

    def test_func(self) -> Optional[bool]:
        return logged_in_as_independent_student(self.request.user)

    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]:
        rapid_router_levels = Level.objects.filter(episode__pk__in=range(1, 10))
        python_den_levels = Level.objects.filter(
            episode__pk__in=[12, 13, 14, 15, 22]
        )
        student = self.request.user.new_student

        return {
            "rapid_router": _compute_scores(student, rapid_router_levels),
            "python_den": _compute_scores(student, python_den_levels),
        }


def _compute_scores(
    student: Student, levels: List[Level] or QuerySet
) -> Dict[str, int]:
    """
    Finds Rapid Router progress and score data for a specific student and a specific
    set of levels. This is used to show quick score data to the student on their
    dashboard.
    :param student: the student whose progress this is looking for
    :param levels: the list of levels to gather the progress data of
    :return: a dictionary of integers:
    - num_completed: number of completed levels. A completed level is a level that has a
    successful attempt (van made it to the final house) regardless of the final score.
    - num_top_scores: number of levels that have been completed with a full final score
    of either 10/10 or 20/20 (depending on whether the level has route score
    or algo score enabled)
    - total_score: the addition of all the completed levels' final scores
    - total_available_score: the addition of the maximum attainable score of all levels
    """
    num_completed = num_top_scores = total_available_score = 0
    total_score = 0.0
    # Get a QuerySet of best attempts for each level
    best_attempts = Attempt.objects.filter(
        level__in=levels, student=student, is_best_attempt=True
    ).select_related("level")

    for level in levels:
        total_available_score += _get_max_score_for_level(level)

    # For each level, compare best attempt's score with level's max score and
    # increment variables as needed
    if best_attempts:
        attempts_dict = {
            best_attempt.level.id: best_attempt
            for best_attempt in best_attempts
        }
        for level in levels:
            attempt = attempts_dict.get(level.id)

            if attempt and attempt.score:
                num_completed += 1
                if attempt.score == _get_max_score_for_level(level):
                    num_top_scores += 1

                total_score += attempt.score

    return {
        "num_completed": num_completed,
        "num_top_scores": num_top_scores,
        "total_score": int(total_score),
        "total_available_score": total_available_score,
    }


def _get_max_score_for_level(level: Level) -> int:
    """
    Calculate max score. A level has a max score of 20 by default unless its
    route score or algorithm score is disable or it is a custom level (not in an
    episode). Levels 1-12 have a max score of 20 even if the algo score is
    disabled.
    :param level: The Rapid Router level to get the max score for.
    :return: the max score of the level.
    """
    return (
        10
        if level.id > 12
        and (
            level.disable_route_score
            or level.disable_algorithm_score
            or not level.episode
        )
        else 20
    )


def username_labeller(request):
    return request.user.username


@login_required(login_url=reverse_lazy("independent_student_login"))
@user_passes_test(
    logged_in_as_independent_student,
    login_url=reverse_lazy("independent_student_login"),
)
def student_join_organisation(request):
    student = request.user.new_student
    request_form = StudentJoinOrganisationForm()

    # check student not managed by a school
    if student.class_field:
        return HttpResponseRedirect(reverse_lazy("student_details"))

    if request.method == "POST":
        if "class_join_request" in request.POST:
            request_form = StudentJoinOrganisationForm(request.POST)
            process_join_organisation_form(request_form, request, student)

        elif "revoke_join_request" in request.POST:
            student.pending_class_request = None
            student.save()
            # Check teacher hasn't since accepted rejection before posting success
            show_cancellation_message_if_student_not_in_class(student, request)
            return HttpResponseRedirect(reverse_lazy("student_edit_account"))

    res = render(
        request,
        "portal/play/student_join_organisation.html",
        {"request_form": request_form, "student": student},
    )
    return res


def process_join_organisation_form(request_form, request, student):
    if request_form.is_valid():
        student.pending_class_request = request_form.klass
        student.save()

        send_dotdigital_email(
            campaign_ids["student_join_request_sent"],
            [student.new_user.email],
            personalization_values={
                "SCHOOL_CLUB_NAME": request_form.klass.teacher.school.name,
                "ACCESS_CODE": request_form.klass.access_code,
            },
        )

        send_dotdigital_email(
            campaign_ids["student_join_request_notification"],
            [student.pending_class_request.teacher.new_user.email],
            personalization_values={
                "USERNAME": student.new_user.username,
                "EMAIL": student.new_user.email,
                "ACCESS_CODE": student.pending_class_request.access_code,
            },
        )

        messages.success(
            request,
            "Your request to join a school has been received successfully.",
        )


def show_cancellation_message_if_student_not_in_class(student, request):
    if not student.class_field:
        messages.success(
            request,
            "Your request to join a school has been cancelled successfully.",
        )



from django import forms


class InviteTeacherForm(forms.Form):

    teacher_first_name = forms.CharField(
        help_text="Enter first name of teacher",
        max_length=100,
        widget=forms.TextInput(attrs={"autocomplete": "off", "placeholder": "First name of teacher"}),
    )
    teacher_last_name = forms.CharField(
        help_text="Enter last name of teacher",
        max_length=100,
        widget=forms.TextInput(attrs={"autocomplete": "off", "placeholder": "Last name of teacher"}),
    )
    teacher_email = forms.EmailField(
        help_text="Enter email address",
        widget=forms.EmailInput(attrs={"autocomplete": "off", "placeholder": "Email address"}),
    )

    make_admin_ticked = forms.BooleanField(
        label="Make an administrator of the school",
        widget=forms.CheckboxInput(),
        initial=False,
        required=False,
    )



from __future__ import absolute_import

import datetime
from functools import wraps

import pytz
from common.models import Teacher, Student
from django.contrib.auth import logout
from django.shortcuts import render
from ratelimit import ALL, UNSAFE

from portal.helpers.ratelimit import is_ratelimited
from portal.helpers.request_handlers import get_access_code_from_request
from portal.templatetags.app_tags import is_logged_in

__all__ = ["ratelimit"]


def ratelimit(
    group=None, key=None, rate=None, method=ALL, block=False, is_teacher=True
):
    """
    Ratelimit decorator, adding custom functionality to django-ratelimit's default
    decorator. On block, the user is logged out, redirected to the "locked out" page,
    and passes in whether the user is a teacher or not, depending on the is_teacher
    argument. The user is blocked and the time at which they are blocked is saved.
    """

    def decorator(fn):
        @wraps(fn)
        def _wrapped(request, *args, **kw):
            old_limited = getattr(request, "limited", False)
            ratelimited = is_ratelimited(
                request=request,
                group=group,
                fn=fn,
                key=key,
                rate=rate,
                method=method,
                increment=True,
            )
            request.limited = ratelimited or old_limited
            if ratelimited and block:
                if is_teacher:
                    model = Teacher
                else:
                    model = Student

                user_to_lockout = None

                lockout_template = "portal/locked_out.html"

                if request.user.is_anonymous:
                    data = request.POST
                    if is_teacher:
                        username = data.get("auth-username")
                    else:
                        username = data.get("username")

                    access_code = get_access_code_from_request(request)
                    model_finder = model.objects.get
                    # look for school student if access code not found
                    # (AttributeError) if student not found (IndexError) move
                    # on to another try block similar logic followed afterwards
                    if access_code:
                        user_to_lockout = model_finder(
                            new_user__first_name=username,
                            class_field__access_code=access_code,  # extract the found text from regex
                        )
                        lockout_template = (
                            "portal/locked_out_school_student.html"
                        )
                    # look for indy student or teacher
                    else:
                        user_to_lockout = model_finder(
                            new_user__username=username
                        )
                else:
                    user_to_lockout = model.objects.get(new_user=request.user)

                if user_to_lockout:
                    user_to_lockout.blocked_time = datetime.datetime.now(
                        tz=pytz.utc
                    )
                    user_to_lockout.save()

                    if is_logged_in(request.user):
                        logout(request)

                    return render(
                        request,
                        lockout_template,
                        {"is_teacher": is_teacher},
                    )

            return fn(request, *args, **kw)

        return _wrapped

    return decorator


ratelimit.ALL = ALL
ratelimit.UNSAFE = UNSAFE



from django.http import HttpResponseRedirect
from django.urls import reverse, reverse_lazy


class MaintenanceMiddleware(object):
    """
    This middleware allows us to turn on "Maintenance Mode". Toggle `MAINTENANCE_MODE` to True in
    `process_view` to redirect all requests in the app to the maintenance holding page.
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        return response

    def process_view(self, request, callback, callback_args, callback_kwargs):
        MAINTENANCE_MODE = False

        if MAINTENANCE_MODE and not request.path.startswith(reverse("maintenance")):
            return HttpResponseRedirect(reverse_lazy("maintenance"))

        if not MAINTENANCE_MODE and request.path.startswith(reverse("maintenance")):
            return HttpResponseRedirect(reverse_lazy("home"))



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [("portal", "0054_pending_join_request_can_be_blank")]

    operations = [
        migrations.AddField(
            model_name="school",
            name="eligible_for_testing",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="userprofile",
            name="preview_user",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 2.0 on 2020-10-28 17:36

import os

from django.contrib.auth.hashers import make_password
from django.db import migrations


def give_portaladmin_teacher_profile(apps, schema_editor):
    """
    This migration is so that we can still log in using the portaladmin User, but from
    the teacher login form, which requires school data to be linked to the User.
    """
    User = apps.get_model("auth", "User")
    UserProfile = apps.get_model("common", "UserProfile")
    School = apps.get_model("common", "School")
    Teacher = apps.get_model("common", "Teacher")
    Class = apps.get_model("common", "Class")
    Student = apps.get_model("common", "Student")

    # Amend portaladmin details
    portaladmin = User.objects.get(username="portaladmin")
    portaladmin.first_name = "Portal"
    portaladmin.last_name = "Admin"
    portaladmin.email = "codeforlife-portal@ocado.com"
    portaladmin.save()

    # Create portaladmin UserProfile
    portaladmin_userprofile = UserProfile.objects.create(user=portaladmin)

    # Find test school
    portaladmin_school, _ = School.objects.get_or_create(
        name="Swiss Federal Polytechnic"
    )

    # Create Teacher object and link it to School
    portaladmin_teacher = Teacher.objects.create(
        title="Mr",
        user=portaladmin_userprofile,
        new_user=portaladmin,
        school=portaladmin_school,
        is_admin=True,
        pending_join_request=None,
    )

    # Create a Class
    portaladmin_class = Class.objects.create(
        name="Portaladmin's class",
        teacher=portaladmin_teacher,
        access_code="PO123",
        classmates_data_viewable=True,
        always_accept_requests=True,
    )

    # Create the student User
    portaladmin_student_user = User.objects.create(
        username="portaladmin student",
        first_name="Portaladmin",
        last_name="Student",
        email="adminstudent@codeforlife.com",
        password=make_password(os.getenv("ADMIN_PASSWORD", "Password1")),
    )

    # Create the student UserProfile
    portaladmin_student_userprofile = UserProfile.objects.create(
        user=portaladmin_student_user
    )

    # Create the Student
    Student.objects.create(
        class_field=portaladmin_class,
        user=portaladmin_student_userprofile,
        new_user=portaladmin_student_user,
        pending_class_request=None,
    )


def revert_portaladmin_data(apps, schema_editor):
    User = apps.get_model("auth", "User")
    UserProfile = apps.get_model("common", "UserProfile")
    Teacher = apps.get_model("common", "Teacher")
    Class = apps.get_model("common", "Class")
    Student = apps.get_model("common", "Student")

    portaladmin = User.objects.get(username="portaladmin")
    portaladmin_userprofile = UserProfile.objects.get(user=portaladmin)
    portaladmin_teacher = Teacher.objects.get(new_user=portaladmin)
    portaladmin_class = Class.objects.get(teacher=portaladmin_teacher)
    portaladmin_student_user = User.objects.get(username="portaladmin student")
    portaladmin_student_userprofile = UserProfile.objects.get(
        user=portaladmin_student_user
    )
    portaladmin_student = Student.objects.get(user=portaladmin_student_userprofile)

    portaladmin_student.delete()
    portaladmin_student_userprofile.delete()
    portaladmin_student_user.delete()
    portaladmin_class.delete()
    portaladmin_teacher.delete()
    portaladmin_userprofile.delete()

    portaladmin.first_name = ""
    portaladmin.last_name = ""
    portaladmin.email = "('codeforlife-portal@ocado.com',)"
    portaladmin.save()


class Migration(migrations.Migration):

    dependencies = [("portal", "0060_delete_guardian")]

    operations = [
        migrations.RunPython(give_portaladmin_teacher_profile, revert_portaladmin_data)
    ]



from __future__ import absolute_import

from selenium.webdriver.common.by import By

from . import onboarding_student_list_page
from .teach_base_page import TeachBasePage


class OnboardingStudentsPage(TeachBasePage):
    def __init__(self, browser):
        super(OnboardingStudentsPage, self).__init__(browser)

        assert self.on_correct_page("onboarding_students_page")

    def create_students(self):
        self._click_create_students()

        return onboarding_student_list_page.OnboardingStudentListPage(self.browser)

    def create_students_empty(self):
        self._click_create_students()

        return self

    def create_students_failure(self):
        self._click_create_students()

        return self

    def _click_create_students(self):
        self.browser.find_element(By.NAME, "new_students").click()

    def type_student_name(self, name):
        self.browser.find_element(By.ID, "id_names").send_keys(name + "\n")
        return self
